function grain = gtCalculateGrain(grain, parameters, varargin)
% GTCALCULATEGRAIN  Calulates diffraction vectors and detector intersection points
%
%     grain = gtCalculateGrain(grain, parameters, varargin)
%     -----------------------------------------------------
%
%     INPUT:
%       grain      = <struct>   grain of interest
%       parameters = <struct>   parameters file
%
%     OPTIONAL INPUT (as a list of pairs):
%       showfigure = <logical>  show or not the figure {false}
%       color      = <logical>  show or not the prediction spot positions coloured as
%                               omega {false}
%       axis       = <logical>  plot axis, image reference and beam direction in the
%                               image {false}
%       overlay    = <double>   difspots image output from "gtPlaceDifspotinFull" (MxM)
%                               M is the size of the overlay image (typically 2048)
%       markersize = <double>   marker size {10}
%       clims      = <double>   scale of grays {[-300 500]}
%       usestrain  = <logical>  use strain info
%
%     OUTPUT:
%       grain      = <struct>   grain of interest
%                               Added fields:
%            .pllab     = <double>
%            .hklsp     = <double>
%            .dvec      = <double>
%            .dvecsam   = <double>
%            .allblobs  = <struct>
%                               Updated fields:
%            .thetatype = <double>
%            .hkl       = <double>
%            .pl        = <double>
%            .theta     = <double>
%            .eta       = <double>
%            .omega     = <double>
%
% Simplified version of gtFedGenerateRandomGrain   15-03-2012  - WLudwig


if nargin < 1
    disp('Usage:  grain =  gtCalculateGrain(grain, parameters, varargin)');
    return
end
if ~exist('parameters','var') || isempty(parameters)
    parameters = load('parameters.mat');
    parameters = parameters.parameters;
end

% set default values for optional arguments
app.color      = false;
app.showfigure = false;
app.axis       = false;
app.markersize = 10;
app.clims      = [-300 500];
app.overlay    = []; % zeros(parameters.acq.ydet, parameters.acq.xdet);
app.usestrain  = false;

app = parse_pv_pairs(app,varargin);

% if ~isfield(grain, 'full') || isempty(grain.full)
%     grain.full = app.overlay;
% end
grain.full = [];

% If strain data is not available, disable strain
if ~isfield(grain,'strain') || ~isfield(grain.strain,'strainT') || ...
    isnan(grain.strain.strainT(1,1))

    app.usestrain = false;
end

% Rodrigues vector (grain orientation)
Rvec   = grain.R_vector;
phase  = grain.phaseid;

acq    = parameters.acq;
cryst  = parameters.cryst(phase);
labgeo = parameters.labgeo;
samgeo = parameters.samgeo;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Detector geometry
beamdir = labgeo.beamdir';
rotdir  = labgeo.rotdir';
detdiru = labgeo.detdiru';
detdirv = labgeo.detdirv';
detrefu = labgeo.detrefu;
detrefv = labgeo.detrefv;
Qdet    = gtFedDetectorProjectionTensor(detdiru, detdirv, 1, 1);
tn      = gtMathsCross(labgeo.detdiru, labgeo.detdirv)';

pixelsize = mean([labgeo.pixelsizeu labgeo.pixelsizev]);

detpos  = (labgeo.detrefpos./pixelsize)';

% the (0,0) corner of the detector in Lab system (edge of detector pixel)
detposcorner = detpos - detrefu*detdiru - detrefv*detdirv;

uvorig  = [labgeo.detrefu, labgeo.detrefv]';
csam    = grain.center' ./ pixelsize';
omstep  = 180/acq.nproj;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Crystallography from parameters file
lambda   = gtConvEnergyToWavelength(acq.energy);
hklsp       = cryst.hklsp;
d0          = cryst.dspacingsp;
thetasp     = cryst.thetasp;
hklt        = cryst.hkl;
thetatypesp = cryst.thetatypesp;

%disp('Translating Miller indices into normalized cartesian coordinates for plane normals')
Bmat = gtCrystHKL2CartesianMatrix(cryst.latticepar);

% Plane normals in Cartesian CRYSTAL coordinates
for ii = size(hklsp, 2):-1:1
    pl_cry(ii, :) = gtCrystHKL2Cartesian(hklsp(:, ii), Bmat)';
end

% Compute all orientation matrices g (Reminder: Vc = g . Vs)
g = gtMathsRod2OriMat(Rvec);

% Express plane normals in cartesian SAMPLE CS
pl_sam = gtVectorCryst2Lab(pl_cry, g);

% Initialse output variables
grain.allblobs.pl    = [];
grain.allblobs.pllab = [];
grain.allblobs.hkl   = [];
grain.allblobs.hklsp = [];
grain.allblobs.thetatype = [];
grain.allblobs.theta = [];
grain.allblobs.sintheta = [];
grain.allblobs.eta   = [];
grain.allblobs.omega = [];
grain.allblobs.dvec  = [];
grain.allblobs.dvecsam = [];
grain.allblobs.uv    = [];
grain.allblobs.uvw   = [];
grain.allblobs.srot  = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute diffraction angles and detector intersection points
rotcomp = gtMathsRotationMatrixComp(rotdir, 'col');

for ii = 1:size(pl_sam, 1)

    % Impose strain on grain
    if (app.usestrain)
        % Deformation tensor (use strain tensor for now)
        defT = grain.strain.strainT;

        % New deformed plane normal and relative elongation (relative d-spacing)
        [pl, drel] = gtStrainPlaneNormals(pl_sam(ii, :)', defT); % unit vector

        % New sin(theta) in deformed state is inversely proportional to elongation
        sinth = sind(thetasp(ii)) / drel;
        theta = asind(sinth);
    else
        pl    = pl_sam(ii, :)';
        theta = thetasp(ii);
        sinth = sind(theta);
    end

    % Four omegas of the plane normal (1-3 and 2-4 are the two Friedel pairs):
    % pls is plus or minus pl, and St is the rotation tensor per each omega
    [om, pllab, pls, St] = gtFedPredictOmega(pl, sinth, beamdir, rotdir, rotcomp);

    % If reflection occurs
    if (~isempty(om))
        % Diffraction vectors % takes coloumn vectors
        dvec = gtFedPredictDiffVecMultiple(pllab, beamdir);

        % Normalize vectors
        dvec_norms = sqrt(sum(dvec .* dvec, 1));
        dvec = dvec ./ dvec_norms([1 1 1], :);

        % Following the convention of the matching output, the omega value
        % smaller than 180deg (spot "a") is used in the pair data.
        % The two Friedel pairs are 1a-1b and 2a-2b.

        if (om(1) < om(3))
            reorder_indx = [1, 3];
        else
            reorder_indx = [3, 1];
        end
        if (om(2) < om(4))
            reorder_indx = [reorder_indx, 2 4];
        else
            reorder_indx = [reorder_indx, 4 2];
        end

        om = om(reorder_indx);
        St = St(:, :, reorder_indx);
        pls = pls(:, reorder_indx);
        pl_lab = pllab(:, reorder_indx);
        dvec = dvec(:, reorder_indx)';
        hkl = hklt(:, thetatypesp(ii))';

        % fill the output structure with the two pairs
        grain.allblobs.pl = [grain.allblobs.pl; pls'];
        grain.allblobs.pllab = [grain.allblobs.pllab; pl_lab'];
        grain.allblobs.hkl = [grain.allblobs.hkl; hkl; hkl; hkl; hkl];
        grain.allblobs.hklsp = [grain.allblobs.hklsp; ...
            hklsp(:, ii)'; -hklsp(:, ii)'; hklsp(:, ii)'; -hklsp(:, ii)'];
        grain.allblobs.sintheta = [grain.allblobs.sintheta; ...
            sinth; sinth; sinth; sinth];
        grain.allblobs.theta = [grain.allblobs.theta; ...
            theta; theta; theta; theta];
        grain.allblobs.thetatype = [grain.allblobs.thetatype; ...
            thetatypesp(ii); thetatypesp(ii); thetatypesp(ii); thetatypesp(ii)];
        grain.allblobs.omega = [grain.allblobs.omega; om'];
        grain.allblobs.dvec = [grain.allblobs.dvec; dvec];
        grain.allblobs.srot = cat(3, grain.allblobs.srot, St);
    end % end if ~empty(om)
end % end for pl_sam

% Eta angles % takes row vector
grain.allblobs.eta = gtGeoEtaFromDiffVec(grain.allblobs.dvec, labgeo);

% Diffraction vector in Sample reference
grain.allblobs.dvecsam = gtGeoLab2Sam(grain.allblobs.dvec, ...
    grain.allblobs.omega, labgeo, samgeo, true);

% u,v,w coordinates on the detector
csam_v = csam(:, ones(1, size(grain.allblobs.srot, 3)));
uvw = gtFedPredictUVWMultiple(grain.allblobs.srot, grain.allblobs.dvec', ...
    csam_v, detpos, tn, Qdet, uvorig, grain.allblobs.omega', omstep);
grain.allblobs.uvw = uvw';
grain.allblobs.uv = uvw(1:2, :)';

% Visual Feedback
if (app.showfigure)

    if (isempty(app.overlay))
        app.overlay = zeros(parameters.acq.ydet, parameters.acq.xdet);
    end

    figure();
    imshow(app.overlay, app.Grayscale);
    hold on

    if (app.axis)
        detusize=acq.xdet;
        detvsize=acq.ydet;

        % Image frame
        plot([0 detusize+1],[0 0],'k')
        plot([0 detusize+1],[detvsize+1 detvsize+1],'k')
        plot([0 0],[0 detvsize+1],'k')
        plot([detusize+1 detusize+1],[0 detvsize+1],'k')

        % Midlines
        plot([0 detusize+1],[detvsize/2+0.5 detvsize/2+0.5],'-.k')
        plot([detusize/2+0.5 detusize/2+0.5],[0 detvsize+1],'-.k')

        % Set figure propeties
        axis equal
        set(gca,'YDir','reverse')
        set(gca,'Position',[0.05 0.05 0.95 0.9])
        xlim([-detusize*0.1 detusize*1.1])
        ylim([-detvsize*0.1 detvsize*1.1])
        xlabel('U direction')
        ylabel('V direction')

        % Beam direction in image
        beamuv = Qdet*beamdir;
        % Arrow in figure indicating beam direction
        quiver(uvorig(1),uvorig(2),beamuv(1),beamuv(2),1000,'-k','Linewidth',3)

        % Rotation axis direction in image
        rotuv = Qdet*rotdir;
        % Arrow in figure indicating rotation axis direction
        quiver(uvorig(1),uvorig(2),rotuv(1),rotuv(2),1000,'-.k','Linewidth',3)
    end

    impixelinfo
    om = grain.allblobs.omega;
    uv = grain.allblobs.uv;

    cmap = jet(1001);
    omtoRGB = @(om) cmap(round((om)/(360)*1000+1),:);

    if (app.color)
        colormap(cmap)
        colorbar
        caxis([0 360])
    else
        colormap(gray);
    end

    for ii = 1 : length(uv)
        if (~isempty(uv(ii,1)))  % there might be no intersection when using offaxis detector configurations (vertical)
            if (app.color)
                plot(uv(ii,1), uv(ii,2), 'or', 'MarkerSize', app.Markersize, ...
                    'MarkerEdgeColor', omtoRGB(om(ii)));
            else
                plot(uv(ii,1), uv(ii,2), 'ob', 'MarkerSize', app.Markersize);
            end
        end
    end
end % showfigure

end % end of function
