function grains_conflicts = gtAnalyseGrainsConflicts(analyse_flag, solve_flag, log_flag, update_flag, thr_num)
% GTANALYSEGRAINSCONFLICTS  Fast automatic conflicts analysis (checks difspots claimed by more than one grain)
%     grains_conflicts = gtAnalyseGrainsConflicts(analyse_flag, solve_flag, log_flag, update_flag, thr_num)
%     -----------------------------------------------------------------------------------------------------
%
%     Reads parameters.mat : cryst -> length(cryst), acq:dir
%           4_grains/spot2grain.mat
%           4_grains/phase_##/index.mat : grain -> length(grain)
%
%           4_grains/conflicts.mat (See below for details)
%
%     Log file is saved as:
%       4_grains/archive/grains_conflicts_DD-MMM-YYYY_N.log    (if log_flag = true)
%
%     INPUT:
%       analyse_flag = <logical>    flag to run the analyser {true}
%       solve_flag   = <logical>    flag to solve automatically conflicts as in
%                                   gtSolveGrainsConflicts {false}
%       log_flag     = <logical>    flag to save output on logfile {true}
%       update_flag  = <logical>    flag to update 'sample.mat' {false}
%       thr_num      = <double>     minimum number of discarder difspots in conflict for a
%                                   grain (those difspots are deselected from the grain) {4}
%
%     OUTPUT:
%       grains_conflicts{phaseid} = <struct>     output for each phase containing info on grains in
%                                                conflicts and their difspots list
%             .grain{grainid}     = <double>     matrix with for each difspotID in common gives
%                                                the grainID and difspotID (2 x M)
%
%             .analyser{grainid}  = <GtAnalyser>
%                     .totspots   = <double>     number of difspots in common with other grains
%                     .idspot     = <double>     list of difspots conflict (2 x totspots)
%                     .totgrains  = <double>     number of grains with which shares difspots
%                     .idconflict = <double>     list of grain ids in conflict
%                                                (1 x totgrains)
%                     .flag       = <logical>    (see gtSolveGrainsConflicts)
%                     .difspotID  = <double>     difspotID from Fsim
%
%             .conflicts{grainid} = <GtConflict> (totgrains x 1)
%                     .difspots   = <cell>       [idconflict, [difspots]]
%                     .maxnum     = <logical>    true if difspots > maxnum
%                     .merge      = <cell>       [idconflict, [difspots]] if maxnum = true
%                     .allequals  = <logical>    true if difspots are the same between the grain
%                                                and idconflict
%                     .delete     = <cell>       [idconflict, [difspots]] if allequals = true
%
%       conflicts.mat contains information read from spot2grain.mat :
%              grain_conflict     = <double>     list of logical if difspot(i) is claimed by more than one grain
%                                                (row vector) (num_phases x num_spots)
%              phase_conflict     = <double>     list of difspots in conflict between phases
%                                                (row vector) (num_phases x num_spots)
%              phase              = <cell>       list of difspotIDs in conflict (column) for each phase
%                                                (num_phases x 1)
%
%
%     Version 005 24-01-2013 by LNervo
%       Changed name from summary to analyser;
%       Changed name from GtGrainAnalyser to GtAnalyser
%       Changed from grains_merge.mat to grains_conflicts.mat
%       Introduced GtGrainConflicts as object for grains
%
%     Version 004 25-09-2012 by LNervo


if ~exist('analyse_flag','var') || isempty(analyse_flag)
    analyse_flag = true;
end
if ~exist('solve_flag','var') || isempty(solve_flag)
    solve_flag = false;
end
if ~exist('log_flag','var') || isempty(log_flag)
    log_flag = true;
end
if ~exist('update_flag','var') || isempty(update_flag)
    update_flag = false;
end
if ~exist('thr_num','var') || isempty(thr_num)
    thr_num = 4;
end


% Start time
tStart = tic;

parameters = [];
load('parameters.mat');
num_phases = length(parameters.cryst);
filedir = fullfile(parameters.acq.dir,'4_grains');
bkpdir  = fullfile(parameters.acq.dir,'4_grains','archive');
[~,msg]=mkdir(bkpdir);disp(msg)

clear parameters

% File names for output; start log file
fname0         = fullfile(bkpdir,'%s');
grains_root    = 'grains_conflicts';
conflicts_root = 'conflicts';

grainFile     = gtLastFileName(sprintf(fname0,[grains_root '_' date() '_']),'new');
conflictsFile = gtLastFileName(sprintf(fname0,[conflicts_root '_' date() '_']),'new');
logname       = strrep(grainFile,'.mat','.log');


% Start recording log text
if (log_flag)
    diary(logname)
end

if (analyse_flag)

    spot2grain = {};
    spot2phase = {};
    filename = fullfile(filedir,'spot2grain.mat');
    load(filename,'spot2grain','spot2phase');
    num_spots = length(spot2grain);

    ngrains = zeros(num_phases,1);
    grain = {};
    for phaseid = 1 : num_phases
        filename = fullfile(filedir, sprintf('phase_%02d', phaseid), 'index.mat');
        load(filename,'grain');
        ngrains(phaseid) = length(grain);
    end
    clear grain

    % Check if a given spsot has been claimed by different phases (phase_conflict)
    phase_conflict = [];
    grain_conflict = zeros(num_phases, num_spots);    % list of problematic difspotids (one list per phase)
    for spotid = 1 : num_spots
        if (length(spot2phase{spotid}) > 1)
            phaseids = unique(spot2phase{spotid});
            if (length(phaseids) > 1)
                phase_conflict = [phase_conflict, spotid];
            else
                grain_conflict(phaseids, spotid) = 1;
            end
        end
    end

    % build list of difspotIDs claimed by more than one grain from a given phase (phase{num_phases})
    phase = cell(num_phases);
    for phaseid = 1 : num_phases
        phase{phaseid} = find(grain_conflict(phaseid, :));
    end

    clear filename num_spots phaseids savename spotid phaseid
    
    if exist(fullfile(filedir,'conflicts.mat'),'file')
        copyfile(fullfile(filedir,'conflicts.mat'), conflictsFile);
    end
    save(fullfile(filedir,'conflicts.mat'),'grain_conflict','phase_conflict','phase');
    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % CREATING grains_conflicts
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% fill grains_conflicts structure for each phaseid .grain{grainid} using
    %%% spot2grain and phase variables

    % initialization of grain/analyser/conflicts to empty cells/Objects
    grains_conflicts = cell(num_phases, 1);
    for phaseid = 1 : num_phases
        grains_conflicts{phaseid} = GtGrainConflicts(ngrains(phaseid));
    end

    % calculation of grains in conflicts with others
    for phaseid = 1 : num_phases
        for ii = 1 : length(phase{phaseid})
            grainids = unique(spot2grain{phase{phaseid}(ii)});
            for jj = 1 : length(grainids)
                othergrains = setdiff(grainids, grainids(jj));
                for kk = 1 : length(othergrains)
                  
                    grains_conflicts{phaseid}.setGrain( grainids(jj),...
                        [grains_conflicts{phaseid}.getGrain(grainids(jj)), ...
                        [othergrains(kk); phase{phaseid}(ii)]] );
                      
                end
            end
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% try to make a analyser of what should have to be done .analyser{grainid}
    
    for phaseid = 1 : num_phases

        grains = grains_conflicts{phaseid}; % GtGrainConflicts object

        for grainid = 1 : ngrains(phaseid)
  
            difspotID = [];
            load(fullfile('4_grains',sprintf('phase_%02d',phaseid),sprintf('grain_%04d.mat',grainid)),'difspotID');
            if ~isempty(difspotID)
                % save difspotID from fsim into grains_conflicts....analyser{}.difspotID
                grains.setDifspotID(grainid, difspotID');
                % set thr_num as max 50 % of total number of difspots used for the
                % reconstruction
                thr_num = min(thr_num, round(length(difspotID)/2));
            else
                gtError('gtAnalyseGrainsConflicts:missing_info',['difspotID is empty in grain_' sprintf('%04d',grainid) ' ... Quitting'])
            end

            % take current grain conflict info
            g = grains.getGrain(grainid);

            if ~isempty(g) % has a conflict
                
                % if it is not empty, g has conflict(s)
                totspots   = size(g,2); % not unique
                idconflict = unique(g(1,:)); % unique IDs of grain in conflict with g
                totgrains  = length(idconflict); % number of grain(s) in conflict with g
                fprintf('grain # %d has %d conflicts:\n', grainid, totgrains)
                disp(idconflict)

                % save info in grains_conflicts
                grains.setTotSpots(grainid, totspots);
                grains.setTotGrains(grainid, totgrains);
                grains.setIdConflict(grainid, idconflict);
                %%% set .conflicts{grainid} for each grain in conflict
                grains.resetConflictsToN(grainid, totgrains);
                % set conflicts of grainid with idconflict list
                grains.compareGrains(grainid, idconflict, thr_num);
                % save IDs of difspots in common from the difspotID list
                grains.buildIdSpot(grainid);

            end % end ~isempty(g)

        end % end for grainid

        % store info in output
        grains_conflicts{phaseid} = grains;

    end % end for phaseid

    if exist(fullfile(filedir,'grains_conflicts.mat'),'file')
        copyfile(fullfile(filedir,'grains_conflicts.mat'),grainFile)
    end
    save(fullfile(filedir,'grains_conflicts.mat'),'grains_conflicts','-v7.3');

end % end if analyse_flag

if (solve_flag)
    sample = gtSolveGrainsConflicts(thr_num, update_flag);
end


tEnd = toc(tStart);
fprintf('Total elapsed time: %0.0f sec\n',tEnd)

diary off

end % end of function
