function proj = gtAstraCreateProjections(volume, proj_geom, proj_size, do_permute, proj_model)
% FUNCTION PROJ = GTASTRACREATEPROJECTIONS  Calculate oblique angle projections of a volume
%     ----------------------------------------------------------
%     Input and output arguments description, variable type (in <>) and default value (in {}) are written below.
%
%     INPUT:
%
%            volume = volume to be projected (<double X, Y, Z>)
%         proj_geom = projection geometry for Astra code (12 element vector created by "gtAstraCreateProjectionGeometry" (<double Nx12>)
%                     [beam_direction(1:3), detect_corner_pos(1:3), detect_u_dir(1:3), detect_v_dir(1:3)]
%         proj_size = vertical and horizontal size of projection image (<double 2>)
%
%     OPTIONAL INPUT:
%
%           permute    = if true, the projection stack will be permuted and
%                        organised as a 4D matrix for visualization (<logical> {false})
%           proj_model = either "parallel3d_vec" (default) or "cone_vec"
%     OUTPUT:
%              proj = 3D matrix of projections images
%                     (proj_size(2), nproj, proj_size(1))
%                     use option permute and "montage(proj)" to display the stack

% try
%    d = gpuDevice;
%    OK = d.SupportsDouble;
% catch mexc1
%    mexc = MException('ASTRA:no_GPU','gtAstraCreateProjections: Submit this job on a GPU machine ! Exiting...');
%    mexc = addCause(mexc, mexc1);
%    throw(mexc)
% end

OK = gtCheckGpu();

if ~ OK
    mexc = MException('ASTRA:no_GPU','gtAstraCreateProjections: Submit this job on a GPU machine ! Exiting...');
    throw(mexc)
end

if isempty(do_permute)
    do_permute = false;
end

if isempty(proj_model)
   proj_model = 'parallel3d_vec';
   sprintf('using %s as projection model', proj_model)
end

hsize = proj_size(2);
vsize = proj_size(1);
nproj = size(proj_geom, 1);

xvol  = size(volume, 1);
yvol  = size(volume, 2);
zvol  = size(volume, 3);


%% projection geometry (type, #rows, #columns, vectors)
proj_geom = astra_create_proj_geom(proj_model, vsize, hsize, proj_geom);

%% volume geometry (x, y, z)
vol_geom = astra_create_vol_geom(xvol, yvol, zvol);

%% create projection data of phantom
proj_id = astra_create_sino3d_cuda(volume, proj_geom, vol_geom);
proj    = astra_mex_data3d('get', proj_id);

if do_permute
    proj = permute(proj, [3 1 2]);
    proj = reshape(proj, vsize, hsize, 1, nproj);
end

astra_mex_data3d('delete', proj_id);

end
