function proj = gtAstraPrepareAbsorptionStack(parameters)
% GTASTRAPREPAREABSORPTIONSTACK  Assembles the 3D projection stack of absorption
%                                images and saves them to file absorption_stack.mat
%
%     proj = gtAstraPrepareAbsorptionStack(parameters)
%     ------------------------------------------------
%     INPUT:
%       parameters <structure>
%
%     OUTPUT:
%       proj <structure> with fields:
%         - stack    <double X,X,Z>  3D stack of absorption images
%         - geom     <double Z,12 >  Vector describing Astra preojection geometry
%         - Omega    <Double Z>      Rotation angles in degrees
%
%
%     Version 002.1 26-09-2012 by AKing
%       Change sign of omega angles (used by 2DFBP) when axis is RH [0 0 1]
%
%     Version 002 26-09-2012 by LNervo
%       Formatting, cleaning, commenting

acq = parameters.acq;
rec = parameters.rec;
geo = parameters.labgeo;

if strcmp(acq.type, '360degree')
        totproj   = 2 * acq.nproj;
        increment = 2 * pi / totproj;
        angle_rad = 0 : increment : 2*pi-increment;
    elseif strcmp(acq.type, '180degree')
        totproj   = acq.nproj;
        increment = pi / totproj;
        angle_rad = pi/2 : increment: 3/2*pi-increment;
    else
        error('Scan type not recognised!')
end

% projection geometry... Here we assume that the absorption images are
% shifted such that the axis of rotation is in the center of the images
% The origin of the sample coordinate system is on the rotation axis - in
% the middle of the illuminated area

% if we try to do this slightly more rigorously, using the refined
% labgeo

% detpos0  = [0, 0, 0]; - assuming abs images (acq.bb) centred on rot axis
pixelsize    = mean([geo.pixelsizeu, geo.pixelsizev]);
% create redundant detorig
geo.detorig = gtGeoDetOrig(geo);
detposcorner = geo.detorig ./ pixelsize;
% from gtForwardSimulate_v2:
detpos0 = detposcorner ...
          + (acq.bb(1) - 1 + acq.bb(3)/2) * geo.detdiru  ...
          + (acq.bb(2) - 1 + acq.bb(4)/2) * geo.detdirv;

rotcomp = gtMathsRotationMatrixComp(geo.rotdir,'row');


% stack = zeros(acq.bb(4),acq.bb(3),totproj/rec.absinterval);
stack = zeros(acq.bb(3), totproj/rec.absinterval/2, acq.bb(4));
geom  = zeros(totproj/rec.absinterval/2, 12);

jj = 1;

fprintf('Building sinogram...');

for ii = totproj / 2 : rec.absinterval : totproj -1

  if acq.interlaced_turns == 0
      absname = fullfile(acq.dir,'1_preprocessing','abs',sprintf('abs%04d.edf',ii));
  else
      absname = fullfile(acq.dir,'1_preprocessing','abs',sprintf('abs_renumbered%04d.edf',ii));
  end

  Omega(jj) = angle_rad(ii + 1);

  stack(:, jj, :)  = -log(edf_read(absname))';
  Rottens = gtMathsRotationTensor(-Omega(jj)*180/pi, rotcomp);
  detdiru = geo.detdiru * Rottens;
  detdirv = geo.detdirv * Rottens;
  detpos  = detpos0 * Rottens;
  r       = geo.beamdir * Rottens;

  geom(jj, :) = [r(1), r(2), r(3), detpos(1), detpos(2), detpos(3), detdiru(1), detdiru(2), detdiru(3), detdirv(1), detdirv(2), detdirv(3)];

  jj = jj + 1;

end

% deal with the sense of the rotation
if all(geo.rotdir==[0 0 1])
    % rotation axis is anti-clockwise
    Omega=-Omega;
end

fprintf('...done!\n')

proj.stack      = stack;
proj.Omega      = Omega;
proj.geom       = geom;
proj.num_rows   = acq.bb(4);
proj.num_cols   = acq.bb(3);
proj.num_iter   = rec.abs_num_iter;
proj.vol_size_x = acq.bb(3);
proj.vol_size_y = acq.bb(3);
proj.vol_size_z = acq.bb(4);


name = fullfile(acq.dir, 'absorption_stack.mat');
disp(['Writing sinogram in ' name]);
save(name, '-struct', 'proj');

end % end of function
