function [sample, grains] = gtSolveGrainsConflicts(thr_num, update_flag)
% GTSOLVEGRAINSCONFLICTS
%     [sample, grains] = gtSolveGrainsConflicts(thr_num, update_flag)
%     ---------------------------------------------------------------
%     To be run in the analysis directory after gtAnalyseGrainsConflicts
%
%     Reads ./4_grains/sample.mat
%           ./4_grains/spot2grain.mat
%           ./4_grains/grains_conflicts.mat
%
%     Writes ./4_grains/out.mat
%        out{phaseid}.deselected_grains     = deselected grains
%        out{phaseid}.modified_grains       = modified grains with analyser{grainid}.totspots < thr_num
%        out{phaseid}.modified_difspots     = corresponding difspots deselected from those grains
%        out{phaseid}.conflicts             = grainid still in conflicts
%        out{phaseid}.conflicts_list        = grainid still in conflicts
%        out{phaseid}.conflicts_unique      = grainid still in conflicts
%        out{phaseid}.spot2grain            = {spot2grain, spot2phase};
%
%     INPUT:
%       thr_num      = <double>     minimum number of spots to be discarded {4}
%       update_flag = <logical>    flag to save 4_grains/out.mat, 
%                                          update 4_grains/sample.mat and 
%                                                 4_grains/spot2grain.mat
%                                  {false}
%
%     OUTPUT:
%       sample = updated sample.mat <struct>
%       grains = updated grains_conflicts.mat <cell>
%
%
%     Version 003 15-06-2012 by LNervo
%       grains_conflicts{phaseid}.analyser{grainid}.flag can be:
%         -2 - completeness < mean - 3*std
%         -1 - threshold = 0
%          0 - no conflict
%          1 - conflict and threshold < 0
%          2 - conflict and boundingBox = [0 0 0 0 0 0]
%          3 - conflict and totspots < thr_num


if ~exist('thr_num','var') || isempty(thr_num)
    thr_num = 4;
end
if ~exist('update_flag','var') || isempty(update_flag)
    update_flag = false;
end

out          = [];
sample       = [];
spot2grain   = [];
spot2phase   = [];
grains_conflicts = [];


bkpdir  = fullfile('4_grains','archive');
fname0  = fullfile(bkpdir,'%s');

sample_root = fullfile('4_grains','sample.mat');
sampleFile  = gtLastFileName(sprintf(fname0,['sample_' date() '_']),'new');

spot_root   = fullfile('4_grains','spot2grain.mat');
spotFile    = gtLastFileName(sprintf(fname0,['spot2grain_' date() '_']),'new');

grains_root = fullfile('4_grains','grains_conflicts.mat');
grainsFile  = gtLastFileName(sprintf(fname0,['grains_conflicts_' date() '_']),'new');

out_root    = fullfile('4_grains','out.mat');
outFile     = gtLastFileName(sprintf(fname0,['out_' date() '_']),'new');

logName     = strrep(outFile,'.mat','.log');

disp(['Log filename in: ' logName])
diary( logName )

load(sample_root);
load(spot_root);
load(grains_root);

num_phases = length(sample.phases);

difspots_list = [];

for phaseid = 1 : num_phases

    redo_rec      = false;
    redo_assemble = false;

    phase  = sample.phases{phaseid};
    grains = grains_conflicts{phaseid};
    
    fprintf('*************************\n')
    fprintf('***     PHASE: %02d     ***\n',phaseid)
    fprintf('*************************\n')

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get grains with threshold < 0 : deselect
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_thr = phase.getGrainsWithFieldValueLessThan('threshold', 0);
    if ~isempty(grainIDs_thr)
        % set phase.selectedGrains = false
        arrayfun(@(array) phase.setSelected(array, false), grainIDs_thr);
        fprintf('  Deselected from sample.mat %d grains with threshold < 0:\n', numel(grainIDs_thr))
        disp(grainIDs_thr)
        arrayfun(@(array) grains.setFlag(array, 1), grainIDs_thr);
        fprintf('  Flag = 1 for %d grains with threshold < 0:\n', numel(grainIDs_thr))
        disp('<<< Check the grain segmentation for those grains >>>')
        redo_rec = false;
        reco_assemble = true;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % bbox empty : deselect
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_bbox0  = find(arrayfun(@(num) ~isempty(intersect(phase.getBoundingBox(num),[0 0 0 0 0 0], 'rows')), 1:length(phase.grains)));
    if ~isempty(grainIDs_bbox0)
        % set phase.selectedGrains = false
        arrayfun(@(array) phase.setSelected(array, false), grainIDs_bbox0);
        fprintf('  Deselected %d grains with boundingBox = [0 0 0 0 0 0]:\n', numel(grainIDs_bbox0))
        disp(grainIDs_bbox0)
        arrayfun(@(array) grains.setFlag(array, 2), grainIDs_bbox0);
        fprintf('  Flag = 2 for %d grains with boundingBox = [0 0 0 0 0 0]:\n', numel(grainIDs_bbox0))
        disp('<<< Check grain segmentation for those grains >>>')
        redo_rec = false;
        reco_assemble = true;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % threshold = 0 : warning
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_thr0 = phase.getGrainsWithFieldValueEqualsTo('threshold', 0);
    if ~isempty(grainIDs_thr0)
        fprintf('  There are %d grains with threshold = 0.\n',numel(grainIDs_thr0))
        if length(grainIDs_thr0) == length(phase.grains)
            disp('  Probably you did not run gtSetupReconstruction yet!')
        else
            disp(grainIDs_thr0)
            arrayfun(@(array) grains.setFlag(array, -1), grainIDs_thr0);
            fprintf('  Flag = -1 for %d grains with threshold = 0\n', numel(grainIDs_thr0))
            disp('<<< Run again gtReconstructGrain and gtChangeThreshold3D for those grains >>>')
            redo_rec = true;
            redo_assemble = true;
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % (completeness - mean ) < 3*std : warning
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    thr_com = mean(phase.completeness(:))-3*std(phase.completeness(:));
    grainIDs_com = phase.getGrainsWithCompletenessLessThan(thr_com);
    if ~isempty(grainIDs_com)
        disp(['  Mean(completeness): ' num2str(mean(phase.completeness(:)))])
        disp(['  Found ' num2str(length(grainIDs_com)) ' grains with completeness < ' num2str(thr_com) ' ( = mean - 3*std ): '])
        disp(grainIDs_com)
        arrayfun(@(array) grains.setFlag(array, -2), grainIDs_com);
        fprintf('  Flag = -2 for %d grains with completeness < mean - 3*std\n', numel(grainIDs_com))
        disp('<<< Please check the goodness of those grains manually. >>>')
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get deselected grains
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_desel = phase.getDeselectedGrains();
    if ~isempty(grainIDs_desel)
        fprintf('  In total deselected %d grains:\n', numel(grainIDs_desel))
        disp(grainIDs_desel)
        % don't care about deselected grains??
        arrayfun(@(array) grains.resetGrainId(array), grainIDs_desel);
        fprintf('  Removed conflicts from %d deselected grains:\n', numel(grainIDs_desel))
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %              GRAINS_CONFLICTS   -   ONLY REMAINING CONFLICTS            %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   handle difspots in conflict from grains{grainid} still in conflicts   %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs = grains.getHasConflictsTrue();
    if ~isempty(grainIDs)
        fprintf('  There are still %d grains with conflict: \n', numel(grainIDs));
        disp(grainIDs)
        totspots = arrayfun(@(array) grains.getTotSpots(array), grainIDs, 'UniformOutput', false);
        difspotID = arrayfun(@(array) grains.getDifspotID(array), grainIDs, 'UniformOutput', false);
        
        check = [];
        for ii=1:length(totspots)
            if totspots{ii} < thr_num
                check{ii} = 'y';
            else
                check{ii} = 'n';
            end
        end
        % ask the user if ok to deselect spots from grains
        check = arrayfun(@(array) inputwdefault(sprintf('Do you want to deselect %d / %d spots from grain # %d? [y/n]',totspots{array}, length(difspotID{array}), ...
                         grainIDs(array)),check{array}), ...
                         1:numel(grainIDs), 'UniformOutput', false);
        check = cellfun(@(array) strcmp(array, 'y'), check, 'UniformOutput', false);
        check = [check{:}];

        if any(check)
            redo_rec = true;
            redo_assemble = true;
        
            grainIDs_mod = grainIDs(check);
            
            % get ids for each grain with difspots in conflict
            idspots = arrayfun(@(array) grains.getIdSpotRow(array, 1), grainIDs_mod, 'UniformOutput', false);
            indexes = arrayfun(@(array) grains.getIdSpotRow(array, 2), grainIDs_mod, 'UniformOutput', false);

            arrayfun(@(num) phase.setSelectedDiffspotsIndexes(grainIDs_mod(num), indexes{num}, false), 1:numel(grainIDs_mod));
            arrayfun(@(num) fprintf('  Deselected %d difspots in sample.mat from grain %d:\n', numel(indexes{num}), grainIDs_mod(num)), 1:numel(grainIDs_mod));
            arrayfun(@(num) disp(idspots{num}), 1:numel(grainIDs_mod));

            % update spot2grain and spot2phase
            output = gtUpdateSpot2Grain(grainIDs_mod, [idspots{:}], spot2grain, spot2phase);
            spot2grain = output.spot2grain;
            spot2phase = output.spot2phase;
            
            conflictid = arrayfun(@(num) grains.getIdConflict(grainIDs_mod(num)), 1:numel(grainIDs_mod), 'UniformOutput', false);

            isThere = arrayfun(@(num) ismember(conflictid{num}, grainIDs_mod), 1:numel(conflictid));
            %conflictid = conflictid(~isThere);
            
            arrayfun(@(num) grains.deselectDifspotsFromGrain(grainIDs_mod(num), idspots{num}, conflictid{num}), 1:numel(grainIDs_mod));

            arrayfun(@(num) fprintf('  Deselected %d difspots in grains_conflicts.mat from grain %d:\n', numel(indexes{num}), grainIDs_mod(num)), 1:numel(grainIDs_mod));
            arrayfun(@(num) disp(idspots{num}), 1:numel(grainIDs_mod));
            arrayfun(@(num) fprintf('  Updated conflicts in grains_conflicts.mat for grain %d:\n', conflictid{num}), 1:numel(conflictid));
            arrayfun(@(num) disp(idspots{num}), 1:numel(conflictid));
        else
            grainIDs_mod = [];
            idspots = [];
        end
        
        grainIDs = grains.getHasConflictsTrue();
        fprintf('  There are still %d grains with conflict: \n', numel(grainIDs));
        disp(grainIDs)
    end

    if update_flag
        copyfile(spot_root, spotFile);
        save(spot_root,'spot2grain','spot2phase');
        disp(['Updated ' spot_root])
    else
        disp('spot2grain.mat has not been saved on disk.')
    end

    % get remaining conflicts
    conflicts = grains.getAllValuesFromCellstrField('analyser','idconflict');

    % saving information
    out{phaseid}.deselected_grains = grainIDs_desel';
    out{phaseid}.modified_grains   = grainIDs_mod;
    out{phaseid}.modified_difspots = idspots;
    out{phaseid}.conflicts_list    = conflicts;
    out{phaseid}.spot2grain        = {spot2grain, spot2phase};

    % update sample structure for current phase
    sample.phases{phaseid} = phase;
    grains_conflicts{phaseid} = grains;

    if redo_rec && redo_assemble && ~isempty(out{phaseid}.modified_grains)
        disp(['  For phase ' num2str(phaseid) ' is recommended to run again the Reconstruction for those grains: '])
        disp(out{phaseid}.modified_grains')
        disp('  and assemble this phase.')
        check = inputwdefault('Do you want to run gtReconstructGrain and gtChangeThreshold3D for those grains? [y/n]','y');
        if strcmpi(check,'y')
            disp('It may take sometimes... Be patient :-)')
            diary off
            for jj = 1 : length(out{phaseid}.modified_grains)
                gtReconstructGrains(out{phaseid}.modified_grains(jj), out{phaseid}.modified_grains(jj), pwd(), phaseid);
                gtChangeThreshold3D(out{phaseid}.modified_grains(jj), out{phaseid}.modified_grains(jj), pwd(), phaseid);
            end
            diary on
            disp('  gtReconstructGrain and gtChangeThreshold3D have been executed for grains:')
            disp(out{phaseid}.modified_grains')
        end
    elseif redo_assemble
        disp(['  For phase ' num2str(phaseid) ' is recommended to run again the Assemble.'])
    end

    if ~update_flag
        disp('Remember that spot2grain.mat has not been updated after the analysis.')
        disp('  You should do that afterward to be consistent:')
        disp('  The updated spot2grain.mat is stored in')
        disp('    4_grains/out.mat:out{phaseid}.spot2grain')
        disp('  The updated sample.mat is the output from this function: don''t loose it!')
        disp('  The updated grains_conflicts.mat is the second output from this function')
    end

end % end for phaseid

if exist(out_root,'file')
    copyfile(out_root, outFile);
end
save(out_root,'out', '-v7.3');
disp(['Saved output in ' out_root])


if update_flag
    copyfile(sample_root, sampleFile);
    sample.saveToFile(sample_root);
    disp(['Saved ' sample_root])
    
    copyfile(grains_root, grainsFile);
    save(grains_root,'grains_conflicts');
    disp(['Saved ' grains_root])
end

diary('off');

if nargout == 2
    grains = grains_conflicts;
else
    assignin('base','sample_mod',sample);
    assignin('base','out',out)
    assignin('base','grains_conflicts_mod',grains_conflicts);
    disp('Assigned to base workspace variables ''sample_mod'',''grains_conflicts_mod'',''out''...')
end


end % end of function
