function data = gtCubicUnitCell(varargin)
% GTCUBICUNITCELL  Draws the cubic unit cell with crystallographic axes
%                  for Euler angles (0,0,0)
%
%     data = gtCubicUnitCell(varargin)
%     --------------------------------
%     Right-handed reference systems for sample and crystal.
%     Draws the unit cell using this convention:
%     Xs <100> // to <100> crystallographic axis (a-axis)
%     Ys <010> // to <010> crystallographic axis (b-axis)
%     Zs <001> // to <001> crystallographic axis (c-axis)
%
%
%     OPTIONAL INPUT (parse by pairs):
%       facecolors   = <double>   RGB color {[]}
%       centered     = <logical>  center the unit cell in the sample ref {false}
%       draw         = <logical>  draw it {true}
%       ratio        = <double>   c/a ratio {1}
%       ind          = <double>   index of face for drawing the patch {0: for all}
%       vertices     = <logical>  draw the vertices points {false}
%       numbers      = <logical>  draw the vertices numbers {false}
%       patch        = <logical>  draw the patch {false}
%       %caxis        = <logical>  draw caxis {false}
%       view         = <double>   axes view {[103 20]}
%       add_vertices = <logical>  add some extra vertices (c/3 c/4 position) {false}
%
%     OUTPUT:
%       data         = <struct>   drawing options
%
%     Please refer to the DCTdoc_Crystal_orientation_descriptions.pdf 
%     in the DCT_Documentation folder for the graintracking code.
%
%     Version 002 26-10-2013 by LNervo


app.facecolors   = [];
app.centered     = false;
app.draw         = true;
app.ratio        = 1;
app.ind          = 0;
app.vertices     = false;
app.numbers      = false;
app.patch        = false;
app.sampleaxes   = false;
app.crystaxes    = false;
app.caxis        = false;
app.view         = [103 20];
app.add_vertices = false;

app = parse_pv_pairs(app, varargin);

c=1;
a=1/app.ratio;

% calculation of vertices
% 

x0 = [0 1 1 0 0]*a;
y0 = [0 0 1 1 0]*a;

if app.centered
    z0 = repmat(-c/2,1,length(x0));
    z1 = repmat(+c/2,1,length(x0));
else
    z0 = zeros(1,length(x0));
    z1 = repmat(c,1,length(x0));
end
vertices = [[x0;y0;z0],[x0;y0;z1]]';
vertices(1,:)=[];
vertices(5,:)=[];

if app.add_vertices
    vertices(end+1,:) = [0 0 z0(1)];
    vertices(end+1,:) = [0 0 z1(1)];

    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/3];
    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/3*2];

    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/4];
    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/4*2];
    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/4*3];
end

caxis(1,:) = [0 0 z0(1)-c/4];
caxis(2,:) = [0 0 z1(1)+c/4];

faces = {[1 2 6 5]; [2 3 7 6]; [3 4 8 7]; [4 1 5 8];...
         [5 6 7 8]; [1 2 3 4]};
hkl   = [1 0 0; 0 1 0; -1 0 0; 0 -1 0;...
         0 0 1; 0 0 -1]';
faces_sides = [1 2 6 5; 2 3 7 6; 3 4 8 7; 4 1 5 8];
faces_end   = [5 6 7 8; 1 2 3 4]; %
faceColors = [ ...
     1 1 0; ...
     0 0 1; ...
     1 0 0; ...
     0 1 1; ...
     1 0 1; ...
     0 1 0];

   
   
   
if ~isempty(app.facecolors)
    faceColors = app.facecolors;
end
if size(faceColors,1)==1
    faceColors = repmat(faceColors, size(faces,1),1);
end

% for crystallographic axes and labels
orig=repmat([0 0 z0(1)],3,1);
xcrys=[0.6;0;0];
ycrys=[0;0.6;0];
zcrys=[z0(1);z0(1);z0(1)+0.6];
lcrys={'a [100]';...
       'b [010]';...
       'c [001]'};
     

if app.draw
    figure;
    % basal planes
    line(x0,y0,z0);
    line(x0,y0,z1);
    hold on;
    % prismatic planes
    for ii=1:length(x0)
        plot3([x0(ii) x0(ii)],[y0(ii) y0(ii)],[z0(ii) z1(ii)], 'b-', 'LineWidth', 1)
    end
    if app.caxis
        line([0 0],[0 0],[z0(1)-c/4 z1(1)+c/4])
    end
    % cryst axes
    if app.crystaxes
        quiver3(0,0,z0(1),0.6,0,0,0,'k','LineWidth',2)
        quiver3(0,0,z0(1),0,0.6,0,0,'k','LineWidth',2)
        quiver3(0,0,z0(1),0,0,0.6,0,'k','LineWidth',2)
        text(xcrys-0.1,ycrys+0.1,zcrys,lcrys,'Color',[0 0 0])
    end
    view(app.view)
    if app.sampleaxes
        % sample reference system : Xs, Ys, Zs vectors
        quiver3(-1.5,0,0,3,0,0,0,'r');
        quiver3(0,-1.5,0,0,3,0,0,'g');
        quiver3(0,0,z0(1)-0.5,0,0,(z1(1)-z0(1))+1.5,0,'b');
        text(1.5+0.1,0,0,'Xs','Color',[1 0 0])
        text(0,1.5+0.1,0,'Ys','Color',[0 1 0])
        text(0,0,z1(1)+1.1,'Zs','Color',[0 0 1])
    end
    if app.patch
        if app.ind==0
            p = drawMesh(vertices, faces, ...
                         'facecolor', 'flat', 'FaceVertexCData', faceColors);
        else
            p = drawMesh(vertices, faces(ind,:), ...
                         'facecolor', 'flat', 'FaceVertexCData', faceColors(ind,:));
        end
    end
    axis equal
    set(gca,'Visible','off')

    if app.numbers
        text(vertices(:,1),vertices(:,2),vertices(:,3)+0.1,num2str([1:length(vertices)]'),'Color',[0 0 0])
    end
    if app.vertices
        plot3(vertices(:,1),vertices(:,2),vertices(:,3),'.k')
    end
end

if nargout == 1
    data.vertices    = vertices;
    data.caxis       = caxis;
    data.faces       = faces;
    data.faces_sides = faces_sides;
    data.faces_end   = faces_end;
    data.facecolors  = faceColors;
    data.hkl         = hkl;    
    data.orig        = orig;
    data.xcrys       = xcrys;
    data.ycrys       = ycrys;
    data.zcrys       = zcrys;
    data.lcrys       = lcrys;
    if app.draw
        data.hf   = gcf;
        if app.patch
            data.hp   = p;
        end
    end
end

end % end of function
