function [h_link, p] = gtPlotHexagon(grains, varargin)
% GTPLOTHEXAGON  Makes a figure containing subplots of the unit cells of the 
%                selected grains
%
%     [h_link, p] = gtPlotHexagon(grains, varargin)
%     ---------------------------------------------
%     It does not take into account the grain size and the elastic strain.
%
%     Deals with coordinate systems.  R-vector orientation data in the instrument
%     system: x//beam, z upwards, y towards door.
%     In the reconstructed tomography/grain map data, z is downwards, y//x
%     instruments, x//y instrument. Use tomo_setup flag to use this setup.
%
%     For the reference hexagonal unit cell coordinates see gtHexagonalUnitCell.
%
%     INPUT:
%       grains     = <cell>       grain cell-structures for all the grains (Nx1)
%
%     OPTIONAL INPUT (varargin):
%       ids        = <double>     IDs of grains of interest (1xM)
%       cmap       = <double>     Colour map for all the grains (RGB) (N+1x3) {[1 0 0]}
%       angle      = <double>     rotation around axis to visualize orientations 
%                                 from the section plane as defined in 
%                                 gtShowSampleSurface {0}
%       axis       = <double>     rotation axis {[0 0 1]}
%       translate  = <logical>    flag to apply grain center {false}
%       tomo_setup = <logical>    flag to match tomo reference system
%                                 x-->y y-->x z-->-z {false}
%       reference  = <logical>    flag to plot the reference unit cell for each 
%                                 grain orientation {false}
%       section    = <logical>    flag to draw a section on the yz-plane to follow
%                                 gtShowSampleSurface {false}
%       overlap    = <logical>    flag to overlap the first grain in the list to all the
%                                 other grains {false}
%       caxis      = <logical>    flag to draw cell c-axis for each grain {false}
%       view       = <double>     azimuth and elevation for viewing in 3D (1x2) {[-90 0]}
%       linkp      = <cell>       list of properties to link {'View','Visible'}
%       ratio      = <double>     c/a ratio {2}
%       linecolor  = <double>     RGB for linecolor {[0 0 0]}
%       alpha      = <double>     transparency 0<=alpha<=1 {1}
%
%     OUTPUT:
%       h_link     = <graphics.linkprop>
%       p          = <handle>     patches
%
%
%     Version 004 05-06-2013 by LNervo
%       Added c-axis line;
%       Linked some axes properties and given output as 'h_link'
%
%     Version 003 16-01-2013 by LNervo

%     TO DO LIST: fix bug with facealpha..... if ~= 1, it changes the face color

app.ids        = [];
app.cmap       = [];
app.angle      = 0;
app.axis       = [0 0 1];
app.translate  = false;
app.tomo_setup = false;
app.reference  = false;
app.section    = false;
app.overlap    = false;
app.caxis      = false;
app.view       = [-90 0]; % yz plane x positive into the screen
app.linkp      = {'View','Visible'};
app.ratio      = 2; % c/a
app.linecolor  = [0 0 0];
app.alpha      = 1;

app = parse_pv_pairs(app,varargin);


% keep info on grainID in the first column
if isempty(app.ids)
    app.ids = 1:length(grains);
end
if isempty(app.cmap)
    app.cmap = repmat([1 0 0], length(grains), 3);
    app.showbar = false;
end

if size(app.cmap,1)==1 && size(app.cmap,2)>1
    app.cmap = repmat(app.cmap,length(grains),1);
end
    % remove bkg color
if all(app.cmap(1,:) == [0 0 0])
    app.cmap = app.cmap(2:end,:);
end

grains  = grains(app.ids);
cmap    = app.cmap(app.ids,:);

hf = figure();

% find grid size
if length(grains) > 5
    xdim = 5;
    ydim = ceil(length(grains)/xdim);
else
    xdim = length(grains);
    ydim = 1;
end
set(hf, 'Position', [10 10 200*xdim 250*ydim])

% patch object hexagonal prism
data = gtHexagonalUnitCell('ratio',app.ratio,'draw',false,'centered',true);

if ~app.caxis
    data.caxis = [];
end

if app.translate
    centers = gtIndexAllGrainValues(grains, 'center', [], 1, 1:3);
    sizes   = gtIndexAllGrainValues(grains, 'size_int', [], 1, 1)/1000*0.5; % mm grain equivalent radius
else
    centers = zeros(length(grains), 3);
    sizes   = zeros(length(grains), 1);
end

p = [];
for jj = 1:length(grains)
    caxis = [];
    grainID = grains{jj}.id;
    
    vertices = gtComputeGrainUnitCell(grains{jj}.R_vector, data.vertices, ...
        sizes(jj), centers(jj,:), [], []);
    if app.caxis
        caxis = gtComputeGrainUnitCell(grains{jj}.R_vector, data.caxis, ...
            sizes(jj), centers(jj,:), [], []);
    end
 
    % allow for the two coordinate systems.  Transform from instrument to
    % reconstructed tomo coordinates: x-->y y-->x z-->-z
    if app.tomo_setup
        vertices = [vertices(:,2) vertices(:,1) -vertices(:,3)];
        if app.caxis
            caxis = [caxis(:,2) caxis(:,1) -caxis(:,3)];
        end
    end
    % do the rotate to follow the section plane
    vertices = rotateVectors(vertices, 'angle', app.angle, 'axis', app.axis);
    if app.caxis
        caxis = rotateVectors(caxis, 'angle', app.angle, 'axis', app.axis);
    end
    
    if app.overlap && jj == 1
        vertices_overlap = vertices;
        if app.caxis
            caxis_overlap = caxis;
        else
            caxis_overlap = [];
        end
    end
    
    subplot(ydim, xdim, jj);
    hold(gca, 'on');
    
    % reference hex
    if app.reference && ~app.overlap
        gtPlotGrainUnitCell(data.vertices, data.caxis, data.faces, ...
            [0.5 0.5 0.5], [], app.linecolor, 0.5);
    end
    % if overlap the first grain is used as reference for the others
    if app.overlap && jj ~= 1
        gtPlotGrainUnitCell(vertices_overlap, caxis_overlap, data.faces, ...
            cmap(1,:), [], app.linecolor, 0.5);
    end
    % reference background like the section plane
    if app.section
        pos=min(mean(vertices(data.faces_end(1,:), 1)), mean(vertices(data.faces_end(2,:), 1)));
        section_plane_vertices=[pos 1 1; pos 1 -1; pos -1 1; pos -1 -1];
        gtPlotGrainUnitCell(section_plane_vertices, [], [1 2 4 3], ...
            [0 0 0], [], app.linecolor, 0.4);
    end
    % including the rotation for sectioning
    p{end+1} = gtPlotGrainUnitCell(vertices, caxis, data.faces, ...
            cmap(jj,:), [], app.linecolor, app.alpha);
    
    % a bit of labeling
    title(sprintf('grain %04d', grainID));
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    % view
    view(app.view);

    % activate rotating option
    h = rotate3d(hf);
    set(h,'RotateStyle','box','Enable','on');

    axis vis3d;
    axis equal;
    
    userdata(jj).grain = struct('id', grains{jj}.id, 'center', grains{jj}.center, 'R_vector', grains{jj}.R_vector, ...
                                'cmap', cmap(jj,:), 'caxis', caxis, 'vertices', vertices);

end % end for grains

app.cmap = cmap;

set(hf,'UserData',userdata);

h_link = linkprop(findobj(hf,'type','axes'),app.linkp);

print_structure(app, 'general settings:')

end % end of function
