function fedpars = gtFedSetParameters(fedpars, nbl, defaults)
%     fedpars = gtFedSetParameters(fedpars, nbl, defaults)
%     ----------------------------------------------------
%     Set FED parameters in fedpars structure
%     Optional third input 'defaults' returns default parameters
%
%     Otherwise, use funky GUI gtModifyStructure to get parameters from the
%     user (based on default values)
%

% if default values demanded when calling function
if ~exist('defaults','var') || isempty(defaults)
  defaults=false;
end


% does the user want the default options? - how to handle running compiled?
if ~defaults
  check=inputwdefault('Use default FED parameters? [y/n]', 'n');
  if strcmpi(check, 'y')
    defaults=true;
  end
end


% set default values:

    %%%%%%%%%%%%%%%%% REAL DATA %%%%%%%%%%%%%%%%%
    fedpars.real_data = true;
    %%%%%%%%%%%%%%%%%% GEOMETRY %%%%%%%%%%%%%%%%%
    % sample voxel / detector pixel size ratio
    fedpars.detscaleu=1;
    fedpars.detscalev=1;
    % monochromatic beam
    fedpars.beamchroma='mono';
    %%%%%%%%%%%%%%%%%% LOADING GRAIN %%%%%%%%%%%%%%%%%
    % Size of a volume element (sampling)
    fedpars.auto_gvsize = true;
    fedpars.gvsize = [NaN NaN NaN]; % will be set in gtFedLoadRealGrain
    fedpars.max_elements_grain = 10000; % from PR
    fedpars.ngvsub = [1 1 1];
    %%%%%%%%%%%%%%%%%% LOADING BLOBS %%%%%%%%%%%%%%%%%
    fedpars.loadbl=1:nbl;
    fedpars.usebl=1:nbl;
    fedpars.blext = [15 15 10];
    %%%%%%%%%%%%%%%%%% SOLVER %%%%%%%%%%%%%%%%% 
    fedpars.solver = 'Flow';
    fedpars.filename = 'fedout.mat';
    fedpars.centoffs_correction = false;    % should not exist.
    fedpars.wonly=false;
    % Maximum strain values:
    fedpars.dlim    = 0.009*ones(9,1);      % assumed in solution
    fedpars.sufac(:) = 10;
    fedpars.fbbsize = 0.1;
    fedpars.fbbminsize   = [4 4 4];         % minimum ROI size in solver
    fedpars.dmeancorr = 0;
    fedpars.dindex = [1 1 1 0 0 0 0 0 0];
    fedpars.dstepmode = 'continuous';
    fedpars.dsteplim = 0.0002;              % check with Peter
    fedpars.crude_acceleration = false;
    fedpars.T0strategy = 'B';               % how to calculate TO2 matrix - keep this constant normally
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%% STRATEGY %%%%%%%%%%%%%%%%% 
    % some parameters will be handled in a .strategy field for more efficient
    % solving
    % default to three phases
    fedpars.nphases = 3;                     
    
    %%%%%%%%%%%%%%%%%%% STEP 1 %%%%%%%%%%%%%%%%%  
    fedpars.strategy.s1.niter=100;                   % 100 iterations
    fedpars.strategy.s1.crude_acceleration=false;    % not needed for C code projector
    fedpars.strategy.s1.usebl=1:min(14, nbl);        % use only the first 14 blobs initially
    fedpars.strategy.s1.dindex=[1 1 1 0 0 0 0 0 0];  % solve only for rotation components
    fedpars.strategy.s1.dsteplim = 0.0002*ones(9,1); % maximum deformation step
    fedpars.strategy.s1.dstepmode = 'continuous';    % discrete or continuous steps
    fedpars.strategy.s1.dmeancor = false;            % dmean correction 
    fedpars.strategy.s1.dsmoothing=1;                % smooth deformation results
    fedpars.strategy.s1.fbbsize=0.4;                 % factor for calculating flow bounding box size
    fedpars.strategy.s1.fbbminsize=[4 4 4];          % minimum flow bounding box size
    
    
    %%%%%%%%%%%%%%%%%%% STEP 2 %%%%%%%%%%%%%%%%%  
    % values for the second iteration
    fedpars.strategy.s2.niter=200;
    fedpars.strategy.s2.crude_acceleration=false;
    fedpars.strategy.s2.usebl=1:nbl;
    fedpars.strategy.s2.dindex=[1 1 1 0 0 0 0 0 0];
    fedpars.strategy.s2.dsteplim = 0.0001*ones(9,1);
    fedpars.strategy.s2.dstepmode = 'continuous'; 
    fedpars.strategy.s2.dmeancor = false;
    fedpars.strategy.s2.dsmoothing=1;
    fedpars.strategy.s2.fbbsize=0.3;
    fedpars.strategy.s2.fbbminsize=[4 4 4];         
    
    
    %%%%%%%%%%%%%%%%%%% STEP 3 %%%%%%%%%%%%%%%%%
    fedpars.strategy.s3.niter=100;
    fedpars.strategy.s3.crude_acceleration=false;
    fedpars.strategy.s3.usebl=1:nbl;
    fedpars.strategy.s3.dindex=[1 1 1 0 0 0 0 0 0];  
    fedpars.strategy.s3.dsteplim = 0.00005*ones(9,1);
    fedpars.strategy.s3.dstepmode = 'continuous';
    fedpars.strategy.s3.dmeancor = false;
    fedpars.strategy.s3.dsmoothing=1;                % not a good idea to turn off smoothing, it seems...
    fedpars.strategy.s3.fbbsize=0.2;
    fedpars.strategy.s3.fbbminsize=[4 4 4];  
    
if ~defaults
    % get values from the user
    % static parameters first
    disp('Set the static FED values')
    list{1,1}='real_data';      list{1,2}='Real data (true/false)';
    list{2,1}='detscaleu';      list{2,2}='voxelsize/pixelsize, u';
    list{3,1}='detscalev';      list{3,2}='voxelsize/pixelsize, v';
    list{4,1}='beamchroma';     list{4,2}='beam chomaticity (poly/mono)';
    list{5,1}='auto_gvsize';    list{5,2}='auto element size / voxels';
    list{6,1}='gvsize';         list{6,2}='element size';
    list{7,1}='max_elements_grain';list{7,2}='if auto, max elements per grain';
    list{8,1}='ngvsub';         list{8,2}='mystery thingy';
    list{9,1}='blext';          list{9,2}='pad when reading blobs';
    list{10,1}='solver';        list{10,2}='which solver? (Flow/etc/etc)';
    list{11,1}='filename';      list{11,2}='FED output filename';
    list{12,1}='wonly';         list{12,2}='use Omega variations only (true/false)';
    list{13,1}='dlim';          list{13,2}='maximum allowed deformations [Rx Ry Rz e11 e22 e33 e23 e13 e12]';
    list{14,1}='sufac';         list{14,2}='factor in calculating deformation steps';
    list{15,1}='T0strategy';    list{15,2}='how to calculate T02 matrix (A/B)';
    list{16,1}='nphases';       list{16,2}='How many phases (sets of parameters) in solver?';
    
    fedpars=gtModifyStructure(fedpars, list);
    
    if fedpars.nphases<1
        fedpars.nphases=1;
    end

    % get rid of "excess" default phases
    if fedpars.nphases<3
        for ii=fedpars.nphases+1:3
          fedpars.strategy=rmfield(fedpars.strategy, sprintf('s%d', ii));
        end
    end
    
    list{1,1}='niter';              list{1,2}='Number of iterations in this phase';
    list{2,1}='crude_acceleration'; list{2,2}='Crude acceleration of forward projector?';
    list{3,1}='usebl';              list{3,2}='Which blobs to use?';
    list{4,1}='dindex';             list{4,2}='Which deformation to solve? [Rx Ry Rz e11 e22 e33 e23 e13 e12]';
    list{5,1}='dsteplim';           list{5,2}='Maximum change in deformation in a step';
    list{6,1}='dstepmode';          list{6,2}='dstepmode (discr / continuous)';
    list{7,1}='dmeancorr';          list{7,2}='Do dmean correction (true/false)';
    list{8,1}='dsmoothing';         list{8,2}='Maximum change in deformation in a step';
    list{9,1}='fbbsize';            list{9,2}='Factor for calculating flow bounding box size';
    list{10,1}='fbbminsize';        list{10,2}='Minimum flow bounding box size';
    
        for i=1:fedpars.nphases
        fprintf('Now set the dynamic values for phase %d', i)
        fedpars.strategy.(sprintf('s%d', i))=gtModifyStructure(fedpars.strategy.(sprintf('s%d', i)), list);
        end
    
end

end

