function hkil = gtCart2Hex(uvw, latticepar, convention)
% GTCART2HEX  Converts a plane normal in cartesian coordinates (uvw) to the
%             Miller-Bravais indexes in the direct space (hkil)
%
%     hkil = gtCart2Hex(uvw, latticepar, convention)
%     ----------------------------------------------
%     It follows the X-convention using right handed CS.
%     From (uvw) direction in cartesian space to (hkil) plane normal in hexagonal real space 
%
%     INPUT:
%       uvw        = <double>   plane normal in cartesian coordinates (N,3)
%       latticepar = <double>   lattice parameters (row vector 1,6)
%       convention = <string>   hexagonal unit cell convention {'X'}/'Y'
%
%     OUTPUT:
%       hkil       = <double>   plane normal in hexagonal direct space described by 
%                               Miller-Bravais indexes (N,4)
%
%     Version 002 25-10-2013 by LNervo


if ~exist('convention','var') || isempty(convention)
    convention = 'X';
end

if strcmp(convention, 'X')
% rotates counterclockwise
    uvw = rotateVectors(uvw,'angle',30,'axis',[0 0 1]);
end

hkil = zeros(size(uvw,1), 4);

% change to hexagonal axes
hkil(:,1) = uvw(:,1)*sqrt(3)/2 - uvw(:,2)/2;
hkil(:,2) = uvw(:,2);
hkil(:,3) = -(hkil(:,1) + hkil(:,2));
hkil(:,4) = uvw(:,3);

% allow for unit cell parameters
hkil(:,1:3) = hkil(:,1:3)*latticepar(1);
hkil(:,4)   = hkil(:,4)*latticepar(3);

end % end of function
