/*
 * internal_cell_defs.h
 *
 *  Created on: Jul 11, 2012
 *      Author: vigano
 */

#ifndef INTERNAL_CELL_DEFS_H_
#define INTERNAL_CELL_DEFS_H_

#include <math.h>
#include <omp.h>
#include "mex.h"

# define ROUND_DOWN(x, s) ((x) & ~((s)-1))

#if defined(__AVX__)
# define MEMORY_ALIGN 32
#elif defined(__SSE2__)
# define MEMORY_ALIGN 16
#endif

#include <immintrin.h>

template<typename Type>
class SIMDUnrolling {
public:
  const size_t shift;
  const size_t unrolling;
  const size_t block;

  typedef Type vVvf __attribute__((vector_size(MEMORY_ALIGN))) __attribute__((aligned(MEMORY_ALIGN)));

  SIMDUnrolling(const size_t & _unroll)
  : shift(MEMORY_ALIGN / sizeof(Type)), unrolling(_unroll)
  , block(shift * unrolling)
  { }

  const size_t
  get_unroll(const size_t & tot_size) const
  {
    return ROUND_DOWN(tot_size, block);
  }
};

template<typename Type>
class Coeff {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  static const vVvf get(const Type & coeff);
};

template<>
class Coeff<float> {
public:
  typedef SIMDUnrolling<float>::vVvf vVvf;

  static const vVvf get(const float & coeff)
  {
#if defined(__AVX__)
    return (const vVvf) _mm256_set1_ps(coeff);
#else
    return (const vVvf) _mm_set1_ps(coeff);
#endif
  }
};

template<>
class Coeff<double> {
public:
  typedef SIMDUnrolling<double>::vVvf vVvf;

  static const vVvf get(const double & coeff)
  {
#if defined(__AVX__)
    return (const vVvf) _mm256_set1_pd(coeff);
#else
    return (const vVvf) _mm_set1_pd(coeff);
#endif
  }
};



mxClassID
getMxClassOfCell(const mxArray * cell)
{
  const mwSize numElems = mxGetNumberOfElements(cell);
  if (numElems > 0) {
    const mxArray * const firstArray = mxGetCell(cell, 0);
    const mxClassID firstClass = mxGetClassID(firstArray);
    if (firstClass == mxSINGLE_CLASS || firstClass == mxDOUBLE_CLASS) {
      for(mwIndex count = 1; count < numElems; count++) {
        const mxArray * const array = mxGetCell(cell, count);
        if (firstClass != mxGetClassID(array)) {
          // Damn mixed types!
          return mxUNKNOWN_CLASS;
        }
      }
      // OK it is a float type!
      return firstClass;
    } else {
      // It is still an error!
      return firstClass;
    }
  } else {
    return mxUNKNOWN_CLASS;
  }
}

template<typename Type>
class inner_assign {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData2);
  }
};

template<typename Type>
class inner_pow {
  const double exponent;
public:
  inner_pow(const double & _exp) : exponent(_exp) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return pow(inData2, exponent);
  }
};

template<typename Type>
class inner_sqrt {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return sqrt(inData2); // __builtin_ia32_sqrtpd
  }
  const __v2df
  operator()(const __v2df & inData1, const __v2df & inData2) const throw()
  {
    return _mm_sqrt_pd(inData2);
  }
  const __v4sf
  operator()(const __v4sf & inData1, const __v4sf & inData2) const throw()
  {
    return _mm_sqrt_ps(inData2);
  }
#if defined(__AVX__)
  const __v4df
  operator()(const __v4df & inData1, const __v4df & inData2) const throw()
  {
    return _mm256_sqrt_pd(inData2);
  }
  const __v8sf
  operator()(const __v8sf & inData1, const __v8sf & inData2) const throw()
  {
    return _mm256_sqrt_ps(inData2);
  }
#endif
};

template<typename Type>
class inner_sum {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 + inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 + inData2);
  }
};

template<typename Type>
class inner_prod {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 * inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 * inData2);
  }
};

template<typename Type>
class inner_sub {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 - inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 - inData2);
  }
};

template<typename Type>
class inner_div {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 / inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 / inData2);
  }
};


template<typename Type>
class inner_sum_scale {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_scale(const Type & _scale) : scale(_scale), scale_v(Coeff<Type>::get(_scale)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 + scale * inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 + scale_v * inData2);
  }
protected:
  const Type scale;
  const vVvf scale_v;
};

template<typename Type>
class inner_sum_FISTA_scale {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_FISTA_scale(const Type & _scale) : scale(_scale), scale_v(Coeff<Type>::get(_scale)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 + scale * (inData1 - inData2));
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 + scale_v * (inData1 - inData2));
  }
protected:
  const Type scale;
  const vVvf scale_v;
};

template<typename Type>
class inner_sub_scale {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sub_scale(const Type & _scale) : scale(_scale), scale_v(Coeff<Type>::get(_scale)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 - inData2 * scale);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 - inData2 * scale_v);
  }
protected:
  const Type scale;
  const vVvf scale_v;
};


template<typename Type>
class inner_sum_scalar {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_scalar(const Type & _scalar) : scalar(_scalar), scalar_v(Coeff<Type>::get(_scalar)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData2 + scalar);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData2 + scalar_v);
  }
protected:
  const Type scalar;
  const vVvf scalar_v;
};

template<typename Type>
class inner_prod_scalar {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_prod_scalar(const Type & _scalar) : scalar(_scalar), scalar_v(Coeff<Type>::get(_scalar)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData2 * scalar);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData2 * scalar_v);
  }
protected:
  const Type scalar;
  const vVvf scalar_v;
};

inline void
cell_allocate(mxArray * & outArray, const mxArray * const inArray)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  const mwSize numCellDims = mxGetNumberOfDimensions(inArray);
  const mwSize * cellDims = mxGetDimensions(inArray);

  outArray = mxCreateCellArray(numCellDims, cellDims);

  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++)
  {
    const mxArray * inCell = mxGetCell(inArray, cellIdx);
    const mxClassID type = mxGetClassID(inCell);
    const mwSize numDims = mxGetNumberOfDimensions(inCell);
    const mwSize * dims = mxGetDimensions(inCell);
    mxArray * const outCell = mxCreateNumericArray(numDims, dims, type, mxREAL);
    mxSetCell(outArray, cellIdx, outCell);
  }
}

#if defined(__SSE2__)
template<typename Type, const bool allocate, typename Function>
inline void
cell_iteration_sse(mxArray * & outArray, const mxArray * const inArray,
    const Function & func)
{
  if (allocate) {
    cell_allocate(outArray, inArray);
  }

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const Type * const inData = (const Type *) mxGetData(inCell);
      Type * const outData = (Type *) mxGetData(outCell);

      cell_inner_cycle_sse(outData, outData, inData, numElems, func);
    }
  }
}

template<typename Type, typename Function>
inline void
cell_iteration_copy_sse(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2,
    const Function & func)
{
  cell_allocate(outArray, inArray1);

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const Type * const inData1 = (const Type *) mxGetData(inCell1);
      const Type * const inData2 = (const Type *) mxGetData(inCell2);
      Type * const outData = (Type *) mxGetData(outCell);

      cell_inner_cycle_sse(outData, inData1, inData2, numElems, func);
    }
  }
}

template<typename Type, typename Function>
inline void
cell_inner_cycle_sse(Type * const __restrict outData,
    const Type * const __restrict inData1,
    const Type * const __restrict inData2,
    const mwSize & numElems, Function & func)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 8;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

#pragma omp for nowait
  for(mwIndex elemIdx = 0; elemIdx < simd_unroll.get_unroll(numElems);
      elemIdx += simd_unroll.block)
  {
    const vVvf inV11 = *((const vVvf *)&inData1[elemIdx + 0 * simd_unroll.shift]);
    const vVvf inV12 = *((const vVvf *)&inData1[elemIdx + 1 * simd_unroll.shift]);
    const vVvf inV13 = *((const vVvf *)&inData1[elemIdx + 2 * simd_unroll.shift]);
    const vVvf inV14 = *((const vVvf *)&inData1[elemIdx + 3 * simd_unroll.shift]);

    const vVvf inV21 = *((const vVvf *)&inData2[elemIdx + 0 * simd_unroll.shift]);
    const vVvf inV22 = *((const vVvf *)&inData2[elemIdx + 1 * simd_unroll.shift]);
    const vVvf inV23 = *((const vVvf *)&inData2[elemIdx + 2 * simd_unroll.shift]);
    const vVvf inV24 = *((const vVvf *)&inData2[elemIdx + 3 * simd_unroll.shift]);

    *((vVvf *)&outData[elemIdx + 0 * simd_unroll.shift]) = func(inV11, inV21);
    *((vVvf *)&outData[elemIdx + 1 * simd_unroll.shift]) = func(inV12, inV22);
    *((vVvf *)&outData[elemIdx + 2 * simd_unroll.shift]) = func(inV13, inV23);
    *((vVvf *)&outData[elemIdx + 3 * simd_unroll.shift]) = func(inV14, inV24);

    const vVvf inV31 = *((const vVvf *)&inData1[elemIdx + 4 * simd_unroll.shift]);
    const vVvf inV32 = *((const vVvf *)&inData1[elemIdx + 5 * simd_unroll.shift]);
    const vVvf inV33 = *((const vVvf *)&inData1[elemIdx + 6 * simd_unroll.shift]);
    const vVvf inV34 = *((const vVvf *)&inData1[elemIdx + 7 * simd_unroll.shift]);

    const vVvf inV41 = *((const vVvf *)&inData2[elemIdx + 4 * simd_unroll.shift]);
    const vVvf inV42 = *((const vVvf *)&inData2[elemIdx + 5 * simd_unroll.shift]);
    const vVvf inV43 = *((const vVvf *)&inData2[elemIdx + 6 * simd_unroll.shift]);
    const vVvf inV44 = *((const vVvf *)&inData2[elemIdx + 7 * simd_unroll.shift]);

    *((vVvf *)&outData[elemIdx + 4 * simd_unroll.shift]) = func(inV31, inV41);
    *((vVvf *)&outData[elemIdx + 5 * simd_unroll.shift]) = func(inV32, inV42);
    *((vVvf *)&outData[elemIdx + 6 * simd_unroll.shift]) = func(inV33, inV43);
    *((vVvf *)&outData[elemIdx + 7 * simd_unroll.shift]) = func(inV34, inV44);
  }
#pragma omp for nowait
  for(mwIndex elemIdx = simd_unroll.get_unroll(numElems);
      elemIdx < simd.get_unroll(numElems); elemIdx += simd.block)
  {
    const vVvf inV11 = *((const vVvf *)&inData1[elemIdx]);
    const vVvf inV21 = *((const vVvf *)&inData2[elemIdx]);

    *((vVvf *)&outData[elemIdx]) = func(inV11, inV21);
  }
#pragma omp for nowait
  for(mwIndex elemIdx = simd.get_unroll(numElems); elemIdx < numElems; elemIdx++)
  {
    outData[elemIdx] = func(inData1[elemIdx], inData2[elemIdx]);
  }
}

template<typename Type>
inline Type
cell_square_norm_sse(const mxArray * const inArray)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 16;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

  Type temp_sum = 0;
  vVvf accumul = Coeff<Type>::get(0.0);

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const Type * const inData = (const Type *) mxGetData(inCell);

      const mwSize numRoundElems = simd_unroll.get_unroll(numElems);

#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = 0; count < simd_unroll.get_unroll(numElems);
          count += simd_unroll.block)
      {
        const vVvf in00 = *((vVvf *)&inData[count +  0 * simd_unroll.shift]);
        const vVvf in02 = *((vVvf *)&inData[count +  1 * simd_unroll.shift]);
        const vVvf in04 = *((vVvf *)&inData[count +  2 * simd_unroll.shift]);
        const vVvf in06 = *((vVvf *)&inData[count +  3 * simd_unroll.shift]);
        const vVvf in08 = *((vVvf *)&inData[count +  4 * simd_unroll.shift]);
        const vVvf in10 = *((vVvf *)&inData[count +  5 * simd_unroll.shift]);
        const vVvf in12 = *((vVvf *)&inData[count +  6 * simd_unroll.shift]);
        const vVvf in14 = *((vVvf *)&inData[count +  7 * simd_unroll.shift]);
        const vVvf accumulInternal1 = ( in00 * in00 + in02 * in02
                                      + in04 * in04 + in06 * in06
                                      + in08 * in08 + in10 * in10
                                      + in12 * in12 + in14 * in14 );

        const vVvf in16 = *((vVvf *)&inData[count +  8 * simd_unroll.shift]);
        const vVvf in18 = *((vVvf *)&inData[count +  9 * simd_unroll.shift]);
        const vVvf in20 = *((vVvf *)&inData[count + 10 * simd_unroll.shift]);
        const vVvf in22 = *((vVvf *)&inData[count + 11 * simd_unroll.shift]);
        const vVvf in24 = *((vVvf *)&inData[count + 12 * simd_unroll.shift]);
        const vVvf in26 = *((vVvf *)&inData[count + 13 * simd_unroll.shift]);
        const vVvf in28 = *((vVvf *)&inData[count + 14 * simd_unroll.shift]);
        const vVvf in30 = *((vVvf *)&inData[count + 15 * simd_unroll.shift]);
        const vVvf accumulInternal2 = ( in16 * in16 + in18 * in18
                                      + in20 * in20 + in22 * in22
                                      + in24 * in24 + in26 * in26
                                      + in28 * in28 + in30 * in30 );
        accumul += accumulInternal1 + accumulInternal2;
      }
#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = simd_unroll.get_unroll(numElems);
          count < simd.get_unroll(numElems); count += simd.block)
      {
        const vVvf in00 = *((vVvf *)&inData[count]);
        accumul += in00 * in00;
      }
#pragma omp for reduction(+:temp_sum) nowait
      for(mwIndex count = numRoundElems; count < numElems; count++)
      {
        temp_sum += inData[count] * inData[count];
      }
    }
  }
  const Type * const __restrict accumuld = (Type *)&accumul;
  switch (simd_unroll.shift)
  {
  case 8:
    return (temp_sum + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]
                     + accumuld[4] + accumuld[5] + accumuld[6] + accumuld[7]);
  case 4:
    return (temp_sum + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]);
  case 2:
    return (temp_sum + accumuld[0] + accumuld[1]);
  }
}

template<typename Type>
inline Type
cell_dot_product_sse(const mxArray * const inArray1, const mxArray * const inArray2)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 16;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

  Type temp_sum = 0;
  vVvf accumul = Coeff<Type>::get(0.0);

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const Type * const inData1 = (const Type *) mxGetData(inCell1);
      const Type * const inData2 = (const Type *) mxGetData(inCell2);

#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = 0; count < simd_unroll.get_unroll(numElems);
          count += simd_unroll.block)
      {
        const vVvf in00 = *((vVvf *)&inData1[count +  0 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  0 * simd_unroll.shift]);
        const vVvf in01 = *((vVvf *)&inData1[count +  1 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  1 * simd_unroll.shift]);
        const vVvf in02 = *((vVvf *)&inData1[count +  2 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  2 * simd_unroll.shift]);
        const vVvf in03 = *((vVvf *)&inData1[count +  3 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  3 * simd_unroll.shift]);
        const vVvf in04 = *((vVvf *)&inData1[count +  4 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  4 * simd_unroll.shift]);
        const vVvf in05 = *((vVvf *)&inData1[count +  5 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  5 * simd_unroll.shift]);
        const vVvf in06 = *((vVvf *)&inData1[count +  6 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  6 * simd_unroll.shift]);
        const vVvf in07 = *((vVvf *)&inData1[count +  7 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  7 * simd_unroll.shift]);
        const vVvf accumulInternal1 = ( in00 + in01 + in02 + in03
                                      + in04 + in05 + in06 + in07 );

        const vVvf in08 = *((vVvf *)&inData1[count +  8 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  8 * simd_unroll.shift]);
        const vVvf in09 = *((vVvf *)&inData1[count +  9 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count +  9 * simd_unroll.shift]);
        const vVvf in10 = *((vVvf *)&inData1[count + 10 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count + 10 * simd_unroll.shift]);
        const vVvf in11 = *((vVvf *)&inData1[count + 11 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count + 11 * simd_unroll.shift]);
        const vVvf in12 = *((vVvf *)&inData1[count + 12 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count + 12 * simd_unroll.shift]);
        const vVvf in13 = *((vVvf *)&inData1[count + 13 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count + 13 * simd_unroll.shift]);
        const vVvf in14 = *((vVvf *)&inData1[count + 14 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count + 14 * simd_unroll.shift]);
        const vVvf in15 = *((vVvf *)&inData1[count + 15 * simd_unroll.shift])
                        * *((vVvf *)&inData2[count + 15 * simd_unroll.shift]);
        const vVvf accumulInternal2 = ( in08 + in09 + in10 + in11
                                      + in12 + in13 + in14 + in15 );
        accumul += accumulInternal1 + accumulInternal2;
      }
#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = simd_unroll.get_unroll(numElems);
          count < simd.get_unroll(numElems); count += simd.block)
      {
        accumul += (*((vVvf *)&inData1[count])) * (*((vVvf *)&inData2[count]));
      }
#pragma omp for reduction(+:temp_sum) nowait
      for(mwIndex count = simd.get_unroll(numElems); count < numElems; count++)
      {
        temp_sum += inData1[count] * inData2[count];
      }
    }
  }
  const Type * const __restrict accumuld = (Type *)&accumul;
  switch (simd_unroll.shift)
  {
  case 8:
    return (temp_sum + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]
                     + accumuld[4] + accumuld[5] + accumuld[6] + accumuld[7]);
  case 4:
    return (temp_sum + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]);
  case 2:
    return (temp_sum + accumuld[0] + accumuld[1]);
  }
}
#endif

template<typename Type, const bool allocate, typename Function>
inline void
cell_iteration(mxArray * & outArray, const mxArray * const inArray,
    const Function & func)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  if (allocate) {
    cell_allocate(outArray, inArray);
  }

#pragma omp parallel for
  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * const inCell = mxGetCell(inArray, cellIdx);
    mxArray * const outCell = mxGetCell(outArray, cellIdx);

    const mwSize numElems = mxGetNumberOfElements(inCell);
    const Type * const inData = (const Type *) mxGetData(inCell);
    Type * const outData = (Type *) mxGetData(outCell);

    for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++) {
      outData[elemIdx] = func(outData[elemIdx], inData[elemIdx]);
    }
  }
}

template<typename Type, typename Function>
inline void
cell_iteration_copy(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2, const Function & func)
{
  const mwSize numCells = mxGetNumberOfElements(inArray1);
  cell_allocate(outArray, inArray1);

#pragma omp parallel for
  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
    const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
    mxArray * const outCell = mxGetCell(outArray, cellIdx);

    const mwSize numElems = mxGetNumberOfElements(inCell1);
    const Type * const inData1 = (const Type *) mxGetData(inCell1);
    const Type * const inData2 = (const Type *) mxGetData(inCell2);
    Type * const outData = (Type *) mxGetData(outCell);

    for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++) {
      outData[elemIdx] = func(inData1[elemIdx], inData2[elemIdx]);
    }
  }
}


#endif /* INTERNAL_CELL_DEFS_H_ */
