classdef GtPhase < handle
% GTPHASE Is a class that contains all the information of a given phase. All the
% grains are organized in a performance oriented structure and get/set methods
% are provided, to aid the consistent use of the class.
    properties
        grains           = {};     % List of local grain info (see class GtGrain)
        selectedGrains   = [];     % Booleans that enable/disable grains
        completeness     = [];     % Index of completeness of a grain [0, 1]
        center           = [];     % Center of mass of the grain
        R_vector         = [];     % Orientation (in Rodrigues space) of the grain
        boundingBox      = [];     % Bounding Box: [x y z lx ly lz] ?
        active           = true;   % Boolean that enables/disables the phase
        phaseName        = '';     % Recognisable name for the phase
        volumeFile       = '';     % Path of the mat file which contains the data.
                                   % The syntax is: 'filename:field' where field is
                                   % the field in the mat file which actually contains
                                   % the volume.
        bboxExtremes     = [];
        orderedPositions = {};
    end

    methods
        function obj = GtPhase(name, numberOfGrains)
        % GTPHASE/GTPHASE( name, numberOfGrains )
        % Initializes the phase 'name', to the 'numberOfGrains'
            if (~exist('name', 'var'))
                name = '';
            end
            if (~exist('numberOfGrains', 'var'))
                numberOfGrains = 0;
            end
            obj.phaseName = name;
            obj.initGrains(numberOfGrains);
        end
        
        function initGrains(obj, number)
        % GTPHASE/INITGRAINS( number ) Initializes the phase to the 'number' of
        % grains
            obj.grains    = cell(number,1);
            for ii = 1:number
                obj.grains{ii}    = GtGrain();
            end
            obj.selectedGrains   = true(number, 1);
            obj.completeness     = ones(number, 1);
            obj.center           = zeros(number, 3);
            obj.R_vector         = zeros(number, 3);
            obj.boundingBox      = zeros(number, 6);
            obj.bboxExtremes     = zeros(number, 6);
            obj.orderedPositions = {};
        end

        function pushGrain(obj)
        % GTPHASE/PUSHGRAIN Adds a new empty grain at the end of the list
            obj.grains{end+1}    = GtGrain();
            obj.selectedGrains   = [obj.selectedGrains; true];
            obj.completeness     = [obj.completeness; 1];
            obj.center           = [obj.center; 0 0 0];
            obj.R_vector         = [obj.R_vector; 0 0 0];
            obj.boundingBox      = [obj.boundingBox; 0 0 0 0 0 0];
            obj.bboxExtremes     = [obj.bboxExtremes; 0 0 0 0 0 0];
            obj.orderedPositions = {};
        end

                
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Setter Functions

        % phase fields
       
        function setSelected(obj, grainid, sel)
            obj.selectedGrains(grainid, 1) = sel;
        end

        function setCompleteness(obj, grainid, compl)
            if ((compl < 0) || (compl > 1))
                error('DATA:wrong_parameter', ...
                      ['Completeness should be between [0, 1], but is: ' num2str(compl)]);
            end
            obj.completeness(grainid, 1) = compl;
        end

        function setCenter(obj, grainid, center)
            obj.center(grainid, :) = center;
        end

        function setR_vector(obj, grainid, R_vec)
            obj.R_vector(grainid, :) = R_vec;
        end

        function setBoundingBox(obj, grainid, bbox)
            obj.boundingBox(grainid, :) = bbox;
            obj.bboxExtremes(grainid, :) = [bbox(1:3) (bbox(1:3) + bbox(4:6) -1) ];
        end

        % grains fields
        function setThreshold(obj, grainid, thr)
            obj.grains{grainid}.threshold = thr;
        end

        function setSelectedDiffspots(obj, grainid, selDiffspots)
        % function setSelectedDiffspots(obj, grainid, selDiffspots)
            obj.grains{grainid}.selectedDiffspots = selDiffspots;
        end

        function setSelectedDiffspotsIndexes(obj, grainid, indexes, value)
        % function setSelectedDiffspotsIndexes(obj, grainid, indexes, value)
            obj.grains{grainid}.selectedDiffspots(indexes) = value;
        end
    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Getter Functions

        % phase fields
        
        function sel = getSelected(obj, grainid)
            sel = obj.selectedGrains(grainid, 1);
        end

        function compl = getCompleteness(obj, grainid)
            compl = obj.completeness(grainid, 1);
        end

        function center = getCenter(obj, grainid)
            center = obj.center(grainid, :);
        end

        function center = getCenterPixels(obj, grainid, pixelsize)
            center = obj.center(grainid, :)/pixelsize;
        end

        function R_vec = getR_vector(obj, grainid)
            R_vec = obj.R_vector(grainid, :);
        end

        function bbox = getBoundingBox(obj, grainid)
            bbox = obj.boundingBox(grainid, :);
        end

        function bbox = getBoundingBoxExtremes(obj, grainid)
            bbox = obj.bboxExtremes(grainid, :);
        end

        function shift = getBoundingBoxShift(obj, grainid)
            shift = obj.boundingBox(grainid, 1:3);
        end

        % grains fields
        function thr = getThreshold(obj, grainid)
            thr = obj.grains{grainid}.threshold;
        end

        function selDiffspots = getSelectedDiffspots(obj, grainid)
            selDiffspots = obj.grains{grainid}.selectedDiffspots;
        end

        function grain = getGrain(obj, grainid)
            grain = obj.grains{grainid};
        end

        % get all values from grains
        function allvalues = getAllValuesFromGrainsField(obj, field, uniform)
            grainsArray = [obj.grains{:}];
            allvalues = cellfun(@(name) name, {grainsArray.(field)}, 'UniformOutput', uniform);
        end

        % less than
        function grainIDs = getGrainsWithFieldValueLessThan(obj, field, value)
            allvalues = getAllValuesFromGrainsField(obj, field, true);
            grainIDs = find(allvalues < value);
        end
        
        function grainIDs = getGrainsWithFieldValueEqualsTo(obj, field, value)
            allvalues = getAllValuesFromGrainsField(obj, field, true);
            grainIDs = find(allvalues == value); 
        end

        function grainIDs = getGrainsWithCompletenessLessThan(obj, value)
        % GTPHASE/GETGRAINSWITHCOMPLETENESSLESSTHAN  Gets grains with 
        % (0 < completeness <= value)
            grainIDs = find(obj.completeness <= value & obj.completeness > 0);
        end

        function grainIDs = getGrainsWithDiffspotsNumLessThan(obj, value)
        % GTPHASE/GETGRAINSWITHDIFFSPOTSNUMLESSTHAN  Gets grains with selected
        % difspots number less than value
            grainsArray = [obj.grains{:}];
            numDiffSpots = cellfun(@length, {grainsArray.selectedDiffspots});
            grainIDs = find(numDiffSpots < value);
        end

        function numDiffSpots = getSelectedDiffspotsNum(obj, grainid)
        % GTPHASE/GETSELECTEDDIFFSPOTSNUM  Gets number of selected difspots for
        % a grain
            diffSpots = find(obj.grains{grainid}.selectedDiffspots == true);
            numDiffSpots = length(diffSpots);
        end

        function spotIDs = getSelectedDiffspotsTrue(obj, grainid)
        % GTPHASE/GETSELECTEDDIFFSPOTSTRUE  Gets indexes of selected difspots
        % for a grain
            spotIDs = find(obj.grains{grainid}.selectedDiffspots == true);
        end

        function deselected = getDeselectedGrains(obj)
        % GTPHASE/GETDESELECTEDGRAINS  Gets deselected grains
            deselected = find(obj.selectedGrains == false);
        end
        
        function selected = getSelectedGrains(obj)
        % GTPHASE/GETSELECTEDGRAINS  Gets selected grains
            selected = find(obj.selectedGrains == true);
        end
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Analysis Functions
        
        function buildBBSearchList(obj)
        % buildBBSearchList(obj)
            for ii = 1:size(obj.bboxExtremes, 2)
                [obj.orderedPositions{ii,1:2}] = sort(obj.bboxExtremes(:,ii));
            end
        end

        function matchList = simpleSearchBBtouched(obj, point)
        % matchList = simpleSearchBBtouched(obj, point)
        
            matchList = find(  obj.bboxExtremes(:,1) <= point(1) ...
                             & obj.bboxExtremes(:,2) <= point(2) ...
                             & obj.bboxExtremes(:,3) <= point(3) ...
                             ...
                             & obj.bboxExtremes(:,4) >= point(1) ...
                             & obj.bboxExtremes(:,5) >= point(2) ...
                             & obj.bboxExtremes(:,6) >= point(3) ...
                            );
        end

        function IDs = getNeighbours(obj, grainID, oversize)
        % IDs = getNeighbours(obj, grainID, oversize)
        
            if (~exist('oversize', 'var'))
                oversize = 1;
            end
            numGrains = length(obj.grains);
            replOnes = ones(numGrains, 1);

            halfLengths = obj.boundingBox(:, 4:6) / 2;
            grainHalfLengths = halfLengths(grainID, :);

            maxDistances = halfLengths + grainHalfLengths(replOnes, :);

            centersInPixels = obj.boundingBox(:, 1:3) + halfLengths;
            grainCenter = centersInPixels(grainID, :);
            relativeCenters = abs(centersInPixels - grainCenter(replOnes, :));

            IDs = setdiff(find(all(relativeCenters < (maxDistances * oversize), 2)), grainID);
        end
        
        
        function merge = checkOverlapConflict(obj, id1, id2, pixelsize)
        % function merge = checkOverlapConflict(obj, id1, id2, pixelsize)
        
            merge_value1 = false;
            merge_value2 = false;

            % centers
            c1 = obj.center(id1,:) / pixelsize;
            c2 = obj.center(id2,:) / pixelsize;
            dist = norm(c1-c2);
            b1 = obj.boundingBox(id1, :);
            b2 = obj.boundingBox(id2, :);
            % mean of two volumes : get radius equivalent and get 10% of it
            % as threshold
            thr_dist = mean(prod(b2(4:6)), prod(b1(4:6))) ^ (1/3) / 10;
            %thr_dist = 0.10 * (0.6/2)/pixelsize; % 10% of radius of sample
            if dist < thr_dist
                merge_value1 = true;
            end
            
            % R_vector
            r1 = obj.R_vector(id1,:);
            r2 = obj.R_vector(id2,:);
            %fprintf('R_vector1: %5.4f %5.4f %5.4f\nR_vector2: %5.4f %5.4f %5.4f\n',r1,r2)
            angle = atan2(norm(cross(r1,r2)), dot(r1,r2));
            angle = rad2deg(angle);
            %disp(['angle: ' num2str(angle) ' deg'])
            %disp('thr_angle = 1 deg')

            if angle < 15
                merge_value2 = true;
            end
            
            merge.id1 = id1;
            merge.id2 = id2;
            merge.c1 = c1;
            merge.c2 = c2;
            merge.dist = dist;
            merge.b1 = b1;
            merge.b2 = b2;
            merge.value1 = merge_value1;

            merge.r1 = r1;
            merge.r2 = r2;
            merge.angle = angle;
            merge.value2 = merge_value2;
            
            merge.value = merge_value1 & merge_value2;
        end % end checkOverlapConflict
        
    end % end methods
end % end classdef
