function Srot = gtMathsRotationTensor(om, rotcomp)
% FUNCTION Srot = gtMathsRotationTensor(om, rotcomp)
%
% Computes the rotation tensor given the 3 matrices in Rodrigues' 
% rotation formula. Can be used for computing rotation around an arbitrary
% axis.
%
% Rodrigues's rotation formula describes a linear combination of
% matrices with cos(om) only and sin(om) only dependent terms:
%   Srot = Sconst + Scos*cos(om) + Ssin*sin(om)
%     Srot: is the right side rotation tensor (for row vector):
%      vec_rotated = vec*Srot
%     om  : rotation angle
%
% INPUT
%   om            - rotation angle (in degrees)
%   rotcomp.const - Sconst matrix
%   rotcomp.cos   - Scos matrix
%   rotcomp.sin   - Ssin matrix
%
% OUTPUT
%   Srot          - rotation matrix

if (length(om) == 1)
    Srot = rotcomp.const + rotcomp.cos * cosd(om) + rotcomp.sin * sind(om);
else
    % Expand cos(om)
    cosom  = reshape(cosd(om), 1, []);
    cosomr = cosom(ones(9, 1), :);
    cosomr = reshape(cosomr, 3, []);

    % Expand sin(om)
    sinom  = reshape(sind(om), 1, []);
    sinomr = sinom(ones(9, 1), :);
    sinomr = reshape(sinomr, 3, []);

    % Expand matrices
    ind1 = [1; 2; 3];
    ind2 = ind1(:, ones(1, length(om)));

    constr = rotcomp.const(ind1, ind2);
    cosr   = rotcomp.cos(ind1, ind2);
    sinr   = rotcomp.sin(ind1, ind2);

    % Multiply element-wise
    Srot = constr + cosr .* cosomr + sinr .* sinomr;

    % Reshape final output
    Srot = reshape(Srot, 3, 3, []);
end
% % A simpler but slightly slower version would be:
% ones_om = ones(numel(om), 1);
% 
% cos_om(1, 1, :) = cosd(om);
% sin_om(1, 1, :) = sind(om);
% 
% Srot = rotcomp.const(:, :, ones_om) ...
%     + rotcomp.cos(:, :, ones_om) .* cos_om([1 1 1], [1 1 1], :) ...
%     + rotcomp.sin(:, :, ones_om) .* sin_om([1 1 1], [1 1 1], :);
end
