function statsTable = gtOarGetStats(IDs, vars)
% GTOARGETSTATS  Gets Oar stats for the given job IDs
%     -------------------------------------------------------------------------
%     statsTable = gtOarGetStats(IDs, vars)
%
%     INPUT:
%       IDs        = <int>    OAR job ID numbers
%       vars       = <string> OAR Variables to extract
%
%     OUTPUT:
%       statsTable = <struct> Asked variables stored in a structure
%
%     Version 002 02-07-2012 by LNervo
%       Format subfunction header
%       Improve regexp matching
%       Check var through possible OAR variables
%
%     Version 001 04-04-2012 by Nvigano

    % Creating the job query string
    jobsStr = ['oarstat -f' sprintf(' -j %d', IDs)];
    % Getting the output
    [~, output] = system(jobsStr);

    % Now we chop the input:
    jobsBlocks = regexp(output(1:end-1), '\n\n', 'split');
    statsTable = cell(1, length(jobsBlocks));
    for ii = 1:length(IDs)
        statsTable{ii}.('Job_Id') = IDs(ii);
    end

    %%%%%%%%%%%%%%
    % check vars
    %%%%%%%%%%%%%%
    possible_vars = {...
        'assigned_hostnames', ...
        'assigned_resources', ...
        'command', ...
        'cpuset_name', ...
        'dependencies', ...
        'events', ...
        'exit_code', ...
        'initial_request', ...
        'job_array_id', ...
        'job_array_index', ...
        'jobType', ...
        'launchingDirectory', ...
        'message', ...
        'name', ...
        'owner', ...
        'project', ...
        'properties', ...
        'queue', ...
        'reservation', ...
        'resubmit_job_id', ...
        'scheduledStart', ...
        'startTime', ...
        'state', ...
        'stopTime', ...
        'submissionTime', ...
        'types', ...
        'walltime', ...
        'wanted_resources' };

    if (strcmpi(vars, 'all'))
        vars = possible_vars;
    else
        % Removing not allowed variables
        [newVars, acceptedIDs] = intersect(vars, possible_vars);

        if (length(newVars) ~= length(vars))
            fprintf('Removing not existing variables for OAR:\n   ');
            rejectedIDs = setdiff(1:length(vars), acceptedIDs);
            fprintf(' %s', vars{rejectedIDs});
            fprintf('\n');
        end

        vars = newVars;
    end

    incomplete_logs = false;
    % Now extract info from the output of 'oarstat'
    if (~isempty(statsTable))
        for ii = 1:length(vars)
            stats = regexp(jobsBlocks, [vars{ii} ' = (?<' vars{ii} '>.*?)\n'], 'names', 'once');
            if (isempty(stats))
                warning('gtOarGetStats:no_such_stat', ...
                    [ 'The stat "%s" is not reported in the output from ' ...
                      'oarstat. This might create further errors.\n'], ...
                    vars{ii});
                incomplete_logs = true;
            else
                statsTable = cellfun(@(x1, x2) gtMergeStructs(x1, x2, vars{ii}), statsTable, stats);
            end
        end
    end

    if (incomplete_logs)
        fprintf('\nIncomplete logs for jobs: %s\n\n', sprintf(' %d', IDs));
    end
end % end of function 
