function idout=logviewOar(queryname,typefile,look,debug)
% LOGVIEWOAR  Read log file from OAR
%     idout=logviewOar(queryname,[type],[look],[debug])
%     -------------------------------------------------
%
%     Version 002 17-11-2011 by LNervo
%       Also the jobid can be use as input to look at .err .out files
%
%     Version 001 27-10-2011 by LNervo
%       Read .log file for given function or job array id
%
%       INPUT:
%         queryname = function name <string> 
%                     arrayid <double> (if not known it will ask which one 
%                     from the existing id(s))
%                     jobid <double>
%
%          typefile = type of logfile 
%                     <string> [{'log'} | 'err' | 'out' | 'oar' | 'params']
%
%              look = display file content or not 
%                     <logical> [{true} | false]
%
%             debug = print or not comments [{false} | true]
%
%       OUTPUT:
%           arrayid = array of array id(s) from the function specified with
%                     funcname
%

if ~exist('typefile', 'var') || isempty(typefile)
    typefile = 'log';
end
if ~exist('look', 'var') || isempty(look)
    look = true;
end
if ~exist('debug', 'var') || isempty(debug)
    debug = false;
end

out = GtConditionalOutput(debug);

% Load of parameters file (if exists)
oarparameters = [];
if exist('oarparameters.mat', 'file')
    out.fprintf('Loading the oarparameters file...');
    load('oarparameters.mat');
    out.odisp('done!');
else
    error('OAR:no_parameters_found', ...
          'You must run this function from the directory in which there is the oarparameters.mat file.')
end

ask = false;
if ischar(queryname)
    funcname = queryname;
    ask = true;
    out.odisp(['Looking for function ' funcname])
else
    queryid  = queryname;
    oarStats = gtOarGetStats(queryid, {'name','job_array_id','cpuset_name'});
    funcname = oarStats{1}.('name');
    idarray  = str2double(char(oarStats{1}.('job_array_id')));
    % We match "_[numbers]" because some users have numbers in their usernames
    % (experiments) at the ESRF
    idjob = regexp(oarStats{1}.('cpuset_name'), '_\d+', 'match');
    idjob = str2double(idjob{1}(2:end));
    out.odisp(['Looking for arrayid [' num2str(idarray) '] of the function [' funcname '] with jobid [' num2str(queryid) ']'])
end

if ~isempty(fieldnames(oarparameters))
    names = fieldnames(oarparameters);
    match = strfind(names,funcname);
    idout.fname = {};
    jj = 0;
    % XXX - While loops are terrible and should be avoided like hell
    while (~isempty(match) && (jj < length(names)))
        jj = jj + 1;
        if ~isempty(match{jj})
            fname = names{jj};
            idout.(fname).fname = fname;
            idout.fname{length(idout.fname)+1} = fname;
            out.odisp(['Found a function named ' fname])
            subnames = fieldnames(oarparameters.(fname));
            arrayid = cellfun(@(name) str2double(name(6:end)), subnames);
            if ~isempty(arrayid)
                out.odisp(['Found ' length(arrayid) ' arrayid(s) for ' fname ':'])
                out.odisp(arrayid)
                idout.(fname).arrays = arrayid'; % arrays found for the same function
                idout.(fname).logs   = fullfile('/tmp_14_days','oar','log', ['oar.' fname '.' num2str(arrayid(end)) '.log']);
                oarStats = gtOarGetStats(arrayid, {'job_array_id','cpuset_name','job_array_index','submissionTime','resubmit_job_id'});
            else
                out.odisp(['No array id(s) found for ' fname '!'])
                idout.(fname).arrays = [];
                idout.(fname).logs   = [];
                look = false;
            end
            
            if look
                if ask % idarray is not known
                    disp(['Found ' length(arrayid) ' arrayid(s) for ' fname ':'])
                    disp(arrayid)
                    idarray = inputwdefaultnumeric('Which arrayid do you want to look at? ', arrayid(end));
                end
                [~, ind] = intersect(arrayid, idarray);
                if (~isempty(ind))
                    if strcmpi(typefile,'log') 
                        logname = fullfile('/tmp_14_days','oar','log',['oar.' fname '-' getenv('USER') '_' oarStats{ind}.('resubmit_job_id') '.' num2str(idarray) '.' typefile]);
                    elseif (strcmpi(typefile,'err') || strcmpi(typefile,'out'))
                        logname = fullfile('/tmp_14_days','oar','log',['oar.' fname '-' getenv('USER') '_' oarStats{ind}.('resubmit_job_id') '.' num2str(idjob) '.' typefile]);
                    elseif strcmpi(typefile,'oar') || strcmpi(typefile,'params')
                        logname = [fname '-' getenv('USER') '_' oarStats{ind}.('resubmit_job_id') '.' typefile];
                    end
                    if (exist(logname, 'file'))
                        disp(['Reading file ' logname '...'])
                        disp('-----------------------------------------------------------------------------------------')
                        type(logname);
                        disp('-----------------------------------------------------------------------------------------')
                        idout.(fname).logs = logname;
                        idout.(fname).date = oarStats{ind}.('submissionTime');
                    else
                        fprintf('File not found: %s\n', logname)
                    end
                end
                
            end % end if look
            
            if ~isempty(arrayid)
                out.odisp('**************')
                out.odisp(['Array(s) saved in the oarparameters file for the function ' fname])
                out.odisp(arrayid)
                out.odisp('**************')
            end
        end % end if
    end % end while
    
end % end if

end % end of function




