function structure = build_structure(list, separator, debug)
% BUILD_STRUCTURE  Builds some structure starting from a list template in 'list'
%     structure = build_structure(list, [separator], [debug])
%     -------------------------------------------------------
%     
%     INPUT:
%       list          = <struct>     cell structure with names, description,
%                                    classes and usertypes columns
%       separator     = <string>     separator for sub-structures in list struct {'__'}
%                                    i.e. list.index__strategy --> parameters.index.strategy
%                                      'strategy' is a sub-structure of 'index'.
%                                    PS. list.match_calib --> 'match_calib' is a field of
%                                      'list'. 'calib' is not a sub-structure of 'match'.
%
%     OUTPUT:
%       structure     = <struct>     structure built from 'list'
%       substructures = <string>     array of sub-structures found in each field of
%                                    'structure'
%
%     Version 001 18-11-2011 by LNervo
%       For each sub-structure an empty structure is created properly and
%       the name is given in the output 'substructures'


if ~exist('separator', 'var') || isempty(separator)
    separator = '__';
end
if ~exist('debug', 'var') || isempty(debug)
    debug = false;
end

out = GtConditionalOutput(debug);

structure = [];
field     = struct();

if isstruct(list)
    names    = fieldnames(list);
    newnames = fieldnames(list);
    
    for ll = 1:length(names)
        if ~isempty(strfind(names{ll}, separator))
            tmp = names{ll};
            tmp = strrep(tmp, separator, '.'); % default separator is '__'
            newnames{ll} = tmp;
        end
    end

    for ii = 1:length(names)

        out.odisp(['### structure ' names{ii} ' ###'])
        if ~isstruct(list.(names{ii}))
            for jj = 1:size(list.(names{ii}), 1)

                if strcmpi(list.(names{ii}){jj, 3}, 'char')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = char([]);
                    %out.odisp('Found char...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'single')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = single([]);
                    %out.odisp('Found single...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'double')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = double([]);
                    %out.odisp('Found double...')    
                elseif strcmpi(list.(names{ii}){jj, 3}, 'int8')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = int8([]);
                    %out.odisp('Found int8...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'int16')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = int16([]);
                    %out.odisp('Found int16...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'int32')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = int32([]);
                    %out.odisp('Found int32...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'int64')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = int64([]);
                    %out.odisp('Found int64...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'uint')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = uint([]);
                    %out.odisp('Found uint...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'uint8')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = uint8([]);
                    %out.odisp('Found uint8...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'uint16')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = uint16([]);
                    %out.odisp('Found uint16...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'uint32')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = uint32([]);
                    %out.odisp('Found uint32...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'uint64')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = uint64([]);
                    %out.odisp('Found uint64...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'logical')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = false();
                    %out.odisp('Found logical...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'cell')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = cell(1, 1);
                    %out.odisp('Found cell...')
                elseif strcmpi(list.(names{ii}){jj, 3}, 'struct')
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = struct();
                    out.odisp('Found structure...')
                else
                    field.(names{ii}).(list.(names{ii}){jj, 1}) = NaN;
                end
                
            end % end for loop jj
        else
            out.odisp(['Found structure... Run again the function with input the structure <oldinput>.' names{ii}])
        end % end isstruct(list.(names{ii}))

    end % end for loop ii
    
    for ii = 1:length(names)

        if ~strcmp(names{ii}, newnames{ii})
            newname = textscan(newnames{ii}, '%s', 'delimiter', '.');
            newname = newname{1};
            switch size(newname, 1)
                case 1
                    field.(newname{1}) = field.(names{ii});
                case 2
                    field.(newname{1}).(newname{2}) = field.(names{ii});
                case 3
                    field.(newname{1}).(newname{2}).(newname{3}) = field.(names{ii});
                case 4
                    field.(newname{1}).(newname{2}).(newname{3}).(newname{4}) = field.(names{ii});
                case 5
                    field.(newname{1}).(newname{2}).(newname{3}).(newname{4}).(newname{5}) = field.(names{ii});
            end
            field = rmfield(field, names{ii});
        end

    end % end for loop ii
    
    structure = field;
else
    disp('Please give as input a structure');
    return;
end % end for loop isstruct(list)

end % end of function
