function [field,liststruct] = get_structure(par,substr,saveflag,namelist,list)
% GET_STRUCTURE  Gets the field list for each sub structure of par
%     [field,liststruct] = get_structure(par,substr,[saveflag],[namelist],[list])
%     ---------------------------------------------------------------------------
%     Returns automatically the list cell-structure of 'par' and prints
%     the syntax in a text file to be used for building the parameters
%     afterwards.
%
%     INPUT:
%       par        = <struct>     structure to be analysed
%       substr     = <string>     'par' structure name
%                                 i.e. par = parameters.acq  --> substr = 'acq'
%       saveflag   = <logical>    flag to save for each 'struct' field of 'par' into a file
%                                 "[datasetname]_list_[namelist].txt" {false}
%       namelist   = <string>     name of the current 'par' structure {<substr>}
%       list       = <cell>       pre-defined list of {'fieldname','description','class',usertype}
%                                 for each field (N,4)
%
%     OUTPUT:
%       field      = <cell>     built list of {'name','description','class',usertype}
%                               for each field (M,4)
%                               PS. each substructure name is saved separately. 
%                               i.e. parameters.index.strategy 
%                                     ---> list.index__strategy 
%                                    parameters.index.strategy.b
%                                     ---> list.index__strategy__b
%       liststruct = <cell>     sub-structures info found in 'par' structure
%                               liststruct{k}.name      <string>
%                               liststruct{k}.subnames  <cell>
%                               liststruct{k}.nfields   <double>
%                               liststruct{k}.substruct <logical>
%
%     Usage:
%       [list.acq, tmp] = get_structure(parameters.acq,'acq',true,'acq',list);
%
%
%     Version 002 05-09-2013 by LNervo
%       Commented and better formatted


list_struct = [];
field      = [];

cdir=pwd;
ind=strfind(cdir,'/');
dataset=cdir(ind(end)+1:end);

if ~exist('namelist','var') || isempty(namelist)
    namelist=substr;
end
if ~exist('list','var') || isempty(list)
    list=cell(size(par,1),3); 
end
if ~exist('saveflag','var') || isempty(saveflag)
    saveflag = false;
end

names = fieldnames(par);

[~, names, ~] = findValueIntoCell(names, list(:,1));


kk=0;
for ii=1:length(names)
    if strcmpi(names{ii},list{ii,1})
        field.(substr){ii,1} = list{ii,1};
        field.(substr){ii,2} = list{ii,2};
        if ~isempty(list{ii,3})
            field.(substr){ii,3} = list{ii,3};
        else
            field.(substr){ii,3} = class(par.(names{ii}));  
        end
    else
        field.(substr){ii,1} = names{ii};
        field.(substr){ii,2} = '(?)';
        field.(substr){ii,3} = class(par.(names{ii}));
    end
    
    field.(substr){ii,4} = 1;
    if isstruct(par.(names{ii}))
        disp(['Found structure: ' names{ii}])
        name     = names{ii};
        subnames = fieldnames(par.(names{ii}));
        nof      = length(subnames);
        is_in=false;
        for jj=1:length(subnames)
            if isstruct(par.(names{ii}).(subnames{jj}))
                is_in=true;
            end
        end
        substruct=is_in;
        kk=kk+1;
        list_struct{kk}.name      = name;
        list_struct{kk}.subnames  = subnames;
        list_struct{kk}.nfields   = nof;
        list_struct{kk}.substruct = substruct;
    end
end

if saveflag
    sublist=['list.' namelist];
    basename=[dataset '_list_'];
    filename=[basename namelist '.txt'];
    
    
    fid=fopen(filename,'w');
    for ii=1:length(names)
        if ~isstruct(par.(names{ii}))
            left='%s{end+1,%d} = %s;';
            column1=sprintf(left,sublist,1,['''' char(field.(substr){ii,1}) '''']);
            if size(field.(substr){ii,2},1) > 1
                column2=sprintf(left,sublist,2,'''[Add description]''');
            else
                column2=sprintf(left,sublist,2,['''' char(field.(substr){ii,2}) '''']);
            end
            column3=sprintf(left,sublist,3,['''' char(field.(substr){ii,3}) '''']);
            column4=sprintf(left,sublist,4,num2str(field.(substr){ii,4}));
            
            fprintf(fid,'%s\n%s\n%s\n%s\n',column1,column2,column3,column4);
        end
    end
    disp(['Save list_template in ' filename '.']);
    fclose(fid);
end

if nargout > 0
    field = field.(substr);
    if nargout > 1
        liststruct = list_struct;
    end
end    

end % end of function
