function grain = gtTwinForwardSimulate(phaseID, grain, varargin)
% GTTWINFORWARDSIMULATE  Finds diffraction spots in the database relative to the twin variant
%     grain = gtTwinForwardSimulate(phaseID, grain, varargin)
%     --------------------------------------------------------------------------------------
%
%     INPUT:
%       phaseID          = <int>    phase number {1}
%       grain            = <struct> grain structure of interest
%
%     OPTIONAL INPUT (varargin):
%       save_grain      = <logical> flag to save or not the twins_####.mat
%       variantIDs      = <double>  array of ID for variants {all}
%
%     OUTPUT:
%       grain  = cell structure with fields for each variant:
%                - difstack       <double XxYxN>  stack of normalized difspots
%                - proj_geom      <double Nx12>   input for grain reconstruction / forward projection using roi (hsize, vsize)
%                - proj_geom_full <double Nx12>   input for grain reconstruction / forward projection (full images)
%                - proj_geom_abs  <double Nx12>   input for grain reconstruction / forward projection of extinction spots
%                - ondet          <double N>      indices of those spots (from the list in allblobs) which are expected on the detector
%                - uvw            <double Nx3>    spot positions on the detector
%                - hklsp          <double Nx3(4)> signed hk(i)l of reflection
%                - flag           <int    Nx3>    status flag indicating result of forward simulation
%                - allblobs       <struct>        forward simulation output produced by gtFedGenerateRandomGrain
%                - status         <string 5>      explantation string corresponding to flag
%
%     Version 001 16-10-2012 by LNervo
%
%     SUB-FUNCTIONS:
%[sub]- sfGetRoi

if isdeployed
  global GT_DB
  global GT_MATLAB_HOME
  load('workspaceGlobal.mat');
end

gtDBConnect();

parameters = [];
load('parameters.mat');
acq    = parameters.acq;
samgeo = parameters.samgeo;
labgeo = parameters.labgeo;
fsim   = parameters.fsim;

if ~exist('phaseID','var') || isempty(phaseID)
  phaseID = 1;
end

if isdeployed
  phaseID = str2double(phaseID);
  fsim.display_figure = false;
end

app.save_grain = false;
app.variantIDs = 1:length(grain.g_twins);
app = parse_pv_pairs(app, varargin);


difspotfile = '2_difspot/difspot00001.edf';
if exist(difspotfile, 'file');
  info = edf_info(difspotfile);
else
  info = [];
end

%%% this section should be replaced with a function dealing with different
% versions of parameters names:  fed, v1, v2...
detrefu      = labgeo.detrefu;        % detector u (line direction) reference point (center) in lab
detrefv      = labgeo.detrefv;        % detector v (column direction )reference point (center) in lab
detdiru0     = labgeo.detdiru;        % will use detdiru0 to distinguish from rotated version of this vector
detdirv0     = labgeo.detdirv;
rotdir       = labgeo.rotdir;
beamdir      = labgeo.beamdir;

pixelsize    = mean([labgeo.pixelsizeu, labgeo.pixelsizev]);
detrefpos    = labgeo.detrefpos ./ pixelsize;   % detector reference position in pixels
detposcorner = gtGeoDetOrig(labgeo) ./ pixelsize;  % the (0,0) corner of the detector in Lab system (edge of detector pixel)

rotcomp      = gtMathsRotationMatrixComp(rotdir,'row');
omstep       = 180 / acq.nproj;
imagerange   = round(fsim.omegarange/omstep);   % number of images summed up when searching raw data for a missing spot

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load info from database
fprintf('\nLoading reflections from database...\n');
difspottable = [parameters.acq.name,'difspot'];

% Load difspot data in 'allspots'
mymcmd = sprintf(['SELECT difspotID, CentroidX, CentroidY, CentroidImage, '...
  'BoundingBoxXsize, BoundingBoxYsize, BoundingBoxXorigin, BoundingBoxYorigin '...
  'FROM %s'],difspottable);
[allspots.difspotIDs, allspots.CentroidXs, allspots.CentroidYs, allspots.CentroidImages, allspots.BoundingBoxXsizes, ...
  allspots.BoundingBoxYsizes, allspots.BoundingBoxXorigins, allspots.BoundingBoxYorigins] = mym(mymcmd);


% loop over the variants


for kk=app.variantIDs
    fprintf('\nRunning forward simulation for variant %d ...\n',kk);

    variant = grain.g_twins(kk);
    % assume same center for parent and twin
    graincenter = grain.center ./ pixelsize;    % graincenter contains COM coordinates in pixels
    out.g        = variant.g;
    out.R_vector = variant.R_vector;

    %%% Get the average difspot X and Y sizes for the current spots in the grain,
    %  define theshold values which will be used during the database search later

    dif_Xsize    = mean(allspots.BoundingBoxXsizes);
    dif_Ysize    = mean(allspots.BoundingBoxYsizes);

    % take the biggest spot and add a relative extra margin (multiplicative factor oversize)
    hsize = round(max(allspots.BoundingBoxXsizes)*fsim.oversize);
    % ...same for vertical direction
    vsize = round(max(allspots.BoundingBoxYsizes)*fsim.oversize);

    u = variant.allblobs.uv(:, 1);
    v = variant.allblobs.uv(:, 2);

    [~,ondetector{kk}] = findOnDet([u v]);
    ondet = ondetector{kk};

    % Exclude spots which overlap with segmentation boundingbox
    seg = find(all([( (u + dif_Xsize/2 > parameters.seg.bbox(1)) ...
      & (u - dif_Xsize/2 < parameters.seg.bbox(1) + parameters.seg.bbox(3))), ...
      ( (v + dif_Ysize/2 > parameters.seg.bbox(2)) ...
      & (v - dif_Ysize/2 < parameters.seg.bbox(2) + parameters.seg.bbox(4)))], 2));

    ondet    = setdiff(ondet, seg); % indices of spots intersection active area of detector
    numspots = length(ondet);       % number of exspected diffraction spots

    if isempty(ondet)
      error('Something was wrong during gtTwinCalculateVariants')
    end


  difspotID          = NaN(numspots, 1);
  CentroidX          = NaN(numspots, 1);
  CentroidY          = NaN(numspots, 1);
  CentroidImage      = NaN(numspots, 1);
  BoundingBoxXsize   = NaN(numspots, 1);
  BoundingBoxYsize   = NaN(numspots, 1);
  BoundingBoxXorigin = NaN(numspots, 1);
  BoundingBoxYorigin = NaN(numspots, 1);

  for ii = 1: length(allspots.difspotIDs)
      jj = allspots.difspotIDs(ii);
      difspotID(jj)          = jj;
      CentroidX(jj)          = allspots.CentroidXs(ii);
      CentroidY(jj)          = allspots.CentroidYs(ii);
      CentroidImage(jj)      = allspots.CentroidImages(ii);
      BoundingBoxXsize(jj)   = allspots.BoundingBoxXsizes(ii);
      BoundingBoxYsize(jj)   = allspots.BoundingBoxYsizes(ii);
      BoundingBoxXorigin(jj) = allspots.BoundingBoxXorigins(ii);
      BoundingBoxYorigin(jj) = allspots.BoundingBoxYorigins(ii);
  end


    % We will classify spots with a flag  [1...5]:
    % 1 = 'No intensity at expected spot position'
    % 2 = 'Found intensity in FULL images at predicted position'
    % 3 = 'Found segmented difspot at expected position - but position / size do not match strict criteria'

    flag      = zeros(numspots, 1);                % see above
    difstack  = zeros(hsize, numspots, vsize);     % stack of zero padded diffraction spots (projections used for 3D reconstruction)
    bb        = zeros(numspots, 4);                % difspot BoundingBox from database
    status    = cell(numspots, 1);                 % string corresponding to flag (summarizing result of forward simulation)
    spotid    = zeros(numspots, 1);                % Mysql difspottable difspotID
    check     = zeros(numspots, 2);
    hklsp     = zeros(numspots,size(variant.allblobs.hklsp,2));    % signed hk(i)l indices of reflection
    omega     = zeros(numspots, 1);                % rotation angle in degrees
    proj_geom = zeros(numspots, 12);               % Vector used for ROI grain reconstruction in ASTRA (grain shifted to center of roi volume)
    proj_geom_full = zeros(numspots, 12);          % Vector describing full projection geometry (full images, grain at nominal position in sample volume)
    proj_geom_abs  = zeros(numspots, 12);          % Vector describung absorption image projection geometry

    r         = zeros(numspots, 3);                % direction of diffracted beam used in ASTRA
    r_th      = zeros(numspots, 3);                % theoretically predicted direction of diffracted beam (based on R_vector)
    r_ex      = zeros(numspots, 3);                % estimated direction of diffracted beam, based on grain & difspot COM positions
    U         = zeros(numspots, 1);                % COM u positions of difspots from difspottable
    V         = zeros(numspots, 1);                % COM v positions of difspots from difspottable
    full      = zeros(acq.xdet, acq.ydet);


    %%% Main loop:  examine all hklsp and search corresponding spots / intensity on the detector..
    for ii = 1 : numspots
      currentSpotID = ondet(ii);
      pl      = variant.allblobs.pl(currentSpotID, :);
      theta   = variant.allblobs.theta(currentSpotID);
      om_th   = variant.allblobs.omega(currentSpotID);
      image_n = round(om_th/omstep);
      eta     = variant.allblobs.eta(currentSpotID);
      hklsp(ii,:)  = variant.allblobs.hklsp(currentSpotID, :);

      % Maximum offset (in number of images) from the predicted omega position
      % allow for a 1/sin(eta) dependency of this value - limited to a
      % maximum angular offset of fsim.MaxOmegaOffset degrees: this could be
      % improved...
      thr_max_offset = round( fsim.omegarange/(abs(sind(eta))) /omstep );

      % Look for difspots appying loose search criteria - is there any intensity at the predicted spot
      % position ?  Choose the one which is closest to the expected size

      %if isempty(candidateIDs)
        % no candidates have been found: segmentation / overlap / intensity / ... problem ?

        flag(ii) = 1; % No intensity at expected spot position

        if fsim.check_spot
          % in case bb_size for search BoundingBox (used for check of
          % completeness) is empty, use the same size as the difspot stack
          if isempty(fsim.bb_size)
            bbhsize = hsize;
            bbvsize = vsize;
          else
            bbhsize = fsim.bb_size(1);
            bbvsize = fsim.bb_size(2);
          end

          % Adjust search bbox in order to avoid reading information
          % stemming from regions outside the image border
          bbox_ustart = round(u(currentSpotID) - bbhsize/2);
          bbox_vstart = round(v(currentSpotID) - bbvsize/2);
          bbox_uend   = min(acq.xdet, bbox_ustart + bbhsize);
          bbox_vend   = min(acq.ydet, bbox_vstart + bbvsize);
          bbox(1) = max(1, bbox_ustart);
          bbox(2) = max(1, bbox_vstart);
          bbox(3) = (bbox_uend - bbox_ustart + 1);
          bbox(4) = (bbox_vend - bbox_vstart + 1);
          start_image = max(0, image_n - imagerange);
          end_image   = min(image_n + imagerange, 2*acq.nproj - 1);

          % read in small image stack at predicted position read ROI from
          % raw images - m is the mean value for each image in the stack
          [roi, m] = sfGetRoi(start_image, end_image, acq, bbox);
          [intensity, ~] = max(m);
          test = sum(roi,3)/sum(roi(:))*dif_Xsize*dif_Xsize*dif_Ysize;

          if (intensity > parameters.seg.thr_grow_low)
            flag(ii) = 2; % Found intensity in FULL images at predicted position
          end

          if (fsim.assemble_figure)
            full = gtPlaceSubImage2(test, full, bbox(1), bbox(2), 'summed');
          end % app.assemble_figure
        end % app.check_spot

      %else % ~isempty(candidateIDs)
        %             % in case there are several candidates, take the one which has
        %             % the Area close to the expected Area.  This selection "rule" could be
        %             % improved...
        %
        %             [area, area_index]   = sort(abs(BoundingBoxXsize(candidateIDs) .* BoundingBoxYsize(candidateIDs) - dif_Xsize*dif_Ysize));
        %             [image, image_index] = sort(abs(CentroidImage(candidateIDs) - image_n));
        %             candidateID  = candidateIDs(image_index(1));
        %
        %             U(ii)  = CentroidX(candidateID);
        %             V(ii)  = CentroidY(candidateID);
        %             Z      = CentroidImage(candidateID);
        %             bb(ii, :)  = [BoundingBoxXorigin(candidateID), BoundingBoxYorigin(candidateID), ...
        %                             BoundingBoxXsize(candidateID), BoundingBoxYsize(candidateID)];
        %             spotid(ii) = candidateID;
        %
        %             flag(ii) = 3; % Found a segmented difspot at expected position - we will check if it does match strict criteria
        %             info.dim_1 = bb(ii, 3);
        %             info.dim_2 = bb(ii, 4);
        %             spot = gtGetSummedDifSpot(candidateID, parameters, 1, info);
        %             spot = spot / sum(spot(:)) * dif_Xsize * dif_Xsize * dif_Ysize;   % here we force the intensity to be the same in all spots -
        %
        %             if (fsim.assemble_figure)
        %                 full = gtPlaceSubImage2(spot, full, bb(ii, 1), bb(ii, 2), 'summed');
        %             end
        %
        %             cent_ok    = sqrt((U(ii) - u(currentSpotID)).^2 + (V(ii) - v(currentSpotID)).^2) <= dif_Offset;
        %             %size_ok    = all([between(bb(ii,3), dif_XsizeMin, dif_XsizeMax), between(bb(ii,4), dif_YsizeMin, dif_YsizeMax)]);
        %
        %
        %             check(ii,:)  = cent_ok;%], size_ok];
        %
        %
        %             if any(check(ii,:)), flag(ii) = 4; end % otherwise we keep flag == 3
        %
        %
        % we found a segmentedd spot -  can use the theoretically
        % predicted direction (r_th) or apparent direction (based on
        % COM of grain and spot) for backprojection

        % theoretically prediceted omega angle


        % use theoretically predited directions (default)
        omega(ii)  = om_th;
        Rottens    = gtMathsRotationTensor(-omega(ii), rotcomp);
        % theoretically predicted beam direction
        r(ii,:)    = beamdir*Rottens + 2*sind(theta)*pl;



  %       
  %       
  %       hoffset = round((hsize - bb(ii,3)) / 2);
  %       voffset = round((vsize - bb(ii,4)) / 2);
  %       
  %       spot = gtPlaceSubImage2(spot, zeros(vsize, hsize), hoffset+1, ...
  %         voffset+1, 'summed');
  %       difstack(:, ii, :) = spot';
  %       
  %       % position of Detector
  %       detpos  = detposcorner ...
  %         + (bb(ii,1) - hoffset - 1 + hsize/2) * detdiru0 ...
  %         + (bb(ii,2) - voffset - 1 + vsize/2) * detdirv0;
  %       
  %       
  %       det_abs = detposcorner ...
  %         + (acq.bb(1) - 1 + acq.bb(3)/2) * detdiru0 ...
  %         + (acq.bb(2) - 1 + acq.bb(4)/2) * detdirv0;
  %       
  %       detdiru = detdiru0 * Rottens;
  %       detdirv = detdirv0 * Rottens;
  %       % take into account the coordinate shift corresponding to the offset
  %       % of the reconstructed volume
  %       detpos  = detpos * Rottens - graincenter;
  %       det_abs = det_abs * Rottens;
  %       r_abs   = beamdir * Rottens;
  %       
  %       % Compact version, since they're all <1x3> vectors, so we can read
  %       % them easily
  %       proj_geom(ii, :)     = [r(ii,:), detpos, detdiru, detdirv];
  %       proj_geom_abs(ii, :) = [r_abs, det_abs, detdiru, detdirv];
  %       
  %       % calculate the projection geometry for the full images as well
  %       detposFull = detposcorner + detrefu*detdiru0 + detrefv*detdirv0;
  %       proj_geom_full(ii, :)   = proj_geom(ii,:);
  %       proj_geom_full(ii, 4:6) = (detposFull * Rottens);
  %     %end % if isempty(candidateID)
    end % loop over predicted spot positions

    found_reflections  = length(find(flag > 1));
    completeness       = found_reflections / numspots;

    newspots = length(find(flag == 2));
    included = find(flag == 2);
    selected = find(flag(included) == 3);  % indices of the selected spots in list of included spots

    if (~fsim.check_spot)
      fprintf(['Completeness estimates may be too pessimistic - activate ' ...
        'option ''check_spot'' for more reliable results\n'])
    end

    fprintf('\n Grain number %d:\n', grain.id);

    fprintf(['Forward simulation found intensity at %d out of %d expected ' ...
      'spot positions \n'], found_reflections, numspots);
    fprintf('Completeness estimate:  %f\n', completeness);
    fprintf('%d spots will be used for reconstruction\n', length(selected));


    %%% Now fill in the information for the output
    %  remove reflections for which no segmented spot has been found

    out.included  = flag(included) == 3;
    out.selected  = flag(included) == 4;%>= fsim.selected;  % vector of logicals with pre-selected spots active
    out.id        = grain.id;
    out.parent_R_vector  = grain.R_vector;
    out.phaseid   = phaseID;
    out.center    = graincenter*pixelsize;
    out.ondet     = ondet;
    out.allblobs  = grain.allblobs;
    out.full      = full;
    out.flag      = flag;
    out.status    = status;
    out.check     = check;

    out.difspotID = spotid(included);
    out.omega     = omega(included);
    out.hklsp     = hklsp(included,:);
    out.uvw       = grain.allblobs.uvw(ondet(included),:);
    out.uv_exp    = [U(included), V(included)];
    out.bb        = bb(included,:);
    out.r_th      = r_th(included,:);

    out.completeness    = completeness;
    out.proj.stack      = difstack(:, included, :);
    out.proj.geom       = proj_geom(included,:);
    out.proj.geom_full  = proj_geom_full(included,:);
    out.proj.geom_abs   = proj_geom_abs(included,:);
    out.proj.num_rows   = vsize;
    out.proj.num_cols   = hsize;
    out.proj.vol_size_x = round(hsize / sqrt(2));
    out.proj.vol_size_y = round(hsize / sqrt(2));
    out.proj.vol_size_z = round(vsize / sqrt(2));
    out.proj.num_iter   = parameters.rec.num_iter;
    out.vol             = [];
    out.segbb           = zeros(1,6);
    out.threshold       = 0;
    out.shift(1) = round((acq.bb(3)-hsize)/2 + graincenter(1)) + 1;
    out.shift(2) = round((acq.bb(3)-hsize)/2 + graincenter(2)) + 1;
    out.shift(3) = round((acq.bb(4)-vsize)/2 + graincenter(3)) + 1;

    if fsim.display_figure
      out.full = full;
      gtShowFsim(out,out.phaseid,fsim.Grayscale)
    end
    
    grain.g_twins(kk).out = out;

end % loop over the variants

if app.save_grain
    filename = fullfile(acq.dir, '4_grains', sprintf('phase_%02d',phaseID), ...
    sprintf('twins_%04d.mat', grain.id));
    save(filename, 'grain');
end
    
end % end of function

%%
% sub-functions

function [spot, m] = sfGetRoi(start_image, end_image, acq, bb)
% GTFORWARDSIMULATE/SFGETROI Sums a roi in raw images

nimages = end_image - start_image + 1;
spot    = zeros(bb(4), bb(3), nimages);
m       = zeros(nimages, 1);
centerx = round((bb(3)/2 -5) : (bb(3)/2 +5));
centery = round((bb(4)/2 -5) : (bb(4)/2 +5));

if all([bb(1) > 0, bb(2) > 0, bb(1)+bb(3) < acq.xdet, bb(2)+bb(4) < acq.ydet])
  fullImgsDir = fullfile(acq.dir, '1_preprocessing', 'full');
  indexes = start_image : end_image;
  filename = fullfile(fullImgsDir, sprintf('full%04d.edf', indexes(1)));
  info     = edf_info(filename);
  
  
  for img_n = 1:nimages
    fullImgName = fullfile(fullImgsDir, sprintf('full%04d.edf', indexes(img_n)));
    spot(:, :, img_n) = edf_read(fullImgName, bb, [], info);
    m(img_n) = gtImgMeanValue(spot(centery, centerx, img_n));
  end
end

end

